/*
 * Copyright (C) 2002-2005 
 * Emmanuel Saracco <esaracco@users.labs.libre-entreprise.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "url.h"
#include "cache.h"
#include "web_site.h"
#include "bookmarks.h"
#include "lists.h"
#include "search.h"
#include "tooltips.h"
#include "callbacks.h"
#include "utils.h"
#include "check.h"
#include "timeout.h"
#include "connection.h"

#include "application.h"

#define PROGRESS_LABEL_MAX_LEN 70

extern char **environ;

/* hash table of the status code
   properties (message, bg etc.) */
static GHashTable *uc_application_status_code_properties_hash;

static void uc_application_menu_init (void);
static void uc_application_clean_data (void);
static void uc_application_settings_init (void);
static void uc_application_buttons_display (const GtkWidget * frame,
					    gpointer array,
					    const guint items_size,
					    gpointer selected_array,
					    const guint selected_items_size);
static GList *uc_application_buttons_get_list (const GtkWidget * widget);
static void uc_application_buttons_save_setting (GtkWidget * widget,
						 gchar * path);
static void uc_application_page_information_dialog_display_links (const
								  UCLinkProperties
								  * prop);
static void uc_application_page_information_dialog_display_emails (const
								   UCLinkProperties
								   * prop);
static void uc_application_page_information_dialog_display_header (const
								   UCLinkProperties
								   * prop);
static void uc_application_page_information_dialog_display_meta_tags (const
								      UCLinkProperties
								      * prop);
static void uc_application_settings_dialog_display_filters_list (const gchar *
								 glade,
								 gpointer
								 items,
								 guint
								 items_size,
								 GtkTreeView
								 ** tv,
								 GtkListStore
								 ** model);
static void uc_application_filters_save_settings (GtkTreeView * treeview,
						  const gchar * path);
static void uc_application_status_code_properties_free (void);
static void uc_application_cookie_warning_dialog_not_display_cb (void);
static void uc_application_auth_warning_dialog_not_display_cb (void);
static void uc_application_settings_cookies_toggled_cb (GtkToggleButton *
							button,
							gpointer user_data);

/**
 * uc_application_save_bookmarks:
 * 
 * Show dialog for saving or not bookmarks
 * change.
 */
void
uc_application_save_bookmarks (void)
{
  if (uc_application_dialog_yes_no_show
      (_
       ("Bookmarks modifications will be saved.\n"
	"Your old bookmarks file will be renamed "
	"with \".uc\" extension.\n\n" "Do you want to save?"),
       GTK_MESSAGE_QUESTION))
    uc_bookmarks_save_changes ();
}

/**
 * uc_application_build_url_treeview:
 *
 * Build all needed elements for the main URL tree view.
 */
void
uc_application_build_url_treeview (void)
{
  GtkCellRenderer *prenderer = NULL;
  GtkCellRenderer *trenderer = NULL;
  GtkTreeViewColumn *column = NULL;

  gtk_tree_view_set_model (treeview, GTK_TREE_MODEL (treestore));
  gtk_tree_view_set_headers_visible (treeview, TRUE);

  /* ID column */
  trenderer = gtk_cell_renderer_text_new ();
  gtk_tree_view_insert_column_with_attributes (treeview, -1,
					       NULL, trenderer,
					       "text", UID_COLUMN, NULL);
  column = gtk_tree_view_get_column (treeview, UID_COLUMN);
  gtk_tree_view_column_set_visible (column, FALSE);

  /* LINK STATUS column */
  column = gtk_tree_view_column_new ();

  prenderer = gtk_cell_renderer_pixbuf_new ();
  gtk_tree_view_column_pack_start (column, prenderer, FALSE);
  gtk_tree_view_column_set_attributes (column, prenderer,
				       "pixbuf", LINK_STATUS_ICON_COLUMN,
				       NULL);

  /* LINK TYPE column */
  prenderer = gtk_cell_renderer_pixbuf_new ();
  gtk_tree_view_column_pack_start (column, prenderer, FALSE);
  gtk_tree_view_column_set_attributes (column, prenderer,
				       "pixbuf", LINK_ICON_COLUMN, NULL);

  gtk_tree_view_column_set_resizable (column, TRUE);
  gtk_tree_view_append_column (treeview, column);

  /* LABEL column */
  gtk_tree_view_insert_column_with_attributes (treeview, -1,
					       _("Label"), trenderer,
					       "text", LABEL_COLUMN, NULL);
  column = gtk_tree_view_get_column (treeview, LABEL_COLUMN - 1);
  gtk_tree_view_column_set_sort_column_id (column, LABEL_COLUMN);
  gtk_tree_view_column_set_resizable (column, TRUE);
  gtk_tree_view_column_set_reorderable (column, TRUE);

  /* URL column */
  gtk_tree_view_insert_column_with_attributes (treeview, -1,
					       _("URL"), trenderer,
					       "text", URL_COLUMN, NULL);
  column = gtk_tree_view_get_column (treeview, URL_COLUMN - 1);
  gtk_tree_view_column_set_sort_column_id (column, URL_COLUMN);
  gtk_tree_view_column_set_resizable (column, TRUE);
  gtk_tree_view_column_set_reorderable (column, TRUE);

  /* Last modified column */
  gtk_tree_view_insert_column_with_attributes (treeview, -1,
					       _("Date"), trenderer,
					       "text", LM_COLUMN, NULL);
  column = gtk_tree_view_get_column (treeview, LM_COLUMN - 1);
  gtk_tree_view_column_set_sort_column_id (column, LM_COLUMN);
  gtk_tree_view_column_set_resizable (column, TRUE);
  gtk_tree_view_column_set_reorderable (column, TRUE);

  gtk_tree_view_set_rules_hint (treeview, TRUE);
}

/**
 * uc_application_main_tree_display_all:
 * 
 * Display the whole list of checked url in the
 * main url treeview.
 */
void
uc_application_main_tree_display_all (void)
{
  WSENS (glade_main_window, "display", TRUE);
  gtk_tree_store_clear (treestore);
  uc_check_display_list (uc_lists_checked_links_get (), NULL);
}

/**
 * uc_application_main_tree_display_status:
 * @link_status: the status to display.
 *
 * Only display a list a links with the given status.
 */
void
uc_application_main_tree_display_status (const guint link_status)
{
  GtkTreeIter iter;

  WSENS (glade_main_window, "display", TRUE);
  WSENS (glade_main_window, "link_status", TRUE);
  gtk_tree_store_clear (treestore);
  uc_check_display_list_with_status (uc_lists_checked_links_get (), iter,
				     link_status);
}

/**
 * uc_application_main_tree_display_type:
 * @link_type: the type to display.
 *
 * Only display a list a links with the given type.
 */
void
uc_application_main_tree_display_type (const UCLinkType link_type)
{
  GtkTreeIter iter;

  WSENS (glade_main_window, "display", TRUE);
  WSENS (glade_main_window, "link_type", TRUE);
  gtk_tree_store_clear (treestore);
  uc_check_display_list_with_type (uc_lists_checked_links_get (), iter,
				   link_type);
}

/**
 * uc_application_main_tree_display_protocol:
 * @proto: the protocol to display.
 *
 * Only display a list a links with the given protocol.
 */
void
uc_application_main_tree_display_protocol (const gchar * proto)
{
  GtkTreeIter iter;

  WSENS (glade_main_window, "display", TRUE);
  WSENS (glade_main_window, "protocol", TRUE);
  gtk_tree_store_clear (treestore);
  uc_check_display_list_with_protocol (uc_lists_checked_links_get (), iter,
				       proto);
}

/**
 * uc_application_main_tree_expand_all:
 * 
 * Expand the main url treeview.
 */
void
uc_application_main_tree_expand_all (void)
{
  WSENS (glade_main_window, "display", TRUE);
  gtk_tree_view_expand_all (treeview);
}

/**
 * uc_application_main_tree_collapse_all:
 * 
 * Collapse the main url treeview.
 */
void
uc_application_main_tree_collapse_all (void)
{
  WSENS (glade_main_window, "display", TRUE);
  gtk_tree_view_collapse_all (treeview);
}

/**
 * uc_application_page_information_dialog_display_meta_tags:
 * @UCLinkProperties: URL list node.
 * 
 * Display meta tags.
 */
static void
uc_application_page_information_dialog_display_meta_tags (const
							  UCLinkProperties *
							  prop)
{
  enum
  {
    NAME_COLUMN,
    CONTENT_COLUMN,
    N_COLUMNS
  };
  GtkTreeIter iter;
  GtkTreeView *tv = NULL;
  GtkListStore *model = NULL;
  GtkCellRenderer *renderer = NULL;
  GtkWidget *scroll = NULL;
  GList *item = NULL;

  WGET ("general_scrolled_down", glade_page_information_dialog, scroll);

  model = gtk_list_store_new (N_COLUMNS, G_TYPE_STRING, G_TYPE_STRING);

  item = g_list_first (prop->metas);
  while (item != NULL)
    {
      UCHTMLTag *lp = (UCHTMLTag *) item->data;

      gtk_list_store_append (model, &iter);
      gtk_list_store_set (model, &iter,
			  NAME_COLUMN, (gchar *) lp->label,
			  CONTENT_COLUMN, (gchar *) lp->value, -1);

      item = g_list_next (item);
    }

  tv = (GtkTreeView *) gtk_tree_view_new_with_model ((GtkTreeModel *) model);
  g_object_unref (model), model = NULL;

  renderer = gtk_cell_renderer_text_new ();
  gtk_tree_view_insert_column_with_attributes (tv,
					       -1,
					       _("Name"), renderer,
					       "text", NAME_COLUMN, NULL);
  uc_utils_set_userfriendly_treeview_column (tv, NAME_COLUMN);

  gtk_tree_view_insert_column_with_attributes (tv,
					       -1,
					       _("Content"), renderer,
					       "text", CONTENT_COLUMN, NULL);
  uc_utils_set_userfriendly_treeview_column (tv, CONTENT_COLUMN);

  gtk_tree_view_set_rules_hint (tv, TRUE);

  gtk_container_add (GTK_CONTAINER (scroll), GTK_WIDGET (tv));

  gtk_widget_show_all (scroll);
}

/**
 * uc_application_open_project_dialog_show:
 * 
 * Show the open project dialog.
 */
void
uc_application_open_project_dialog_show (void)
{
  enum
  {
    UID_COLUMN,
    ICON_COLUMN,
    TITLE_COLUMN,
    N_COLUMNS
  };
  GtkTreeIter iter;
  GtkCellRenderer *trenderer = NULL;
  GtkCellRenderer *prenderer = NULL;
  GtkWidget *treeview = NULL;
  GtkTreeViewColumn *column = NULL;
  GList *list = NULL;

  if (uc_project_get_save () &&
      uc_application_dialog_yes_no_show (_("Save the current project?"),
					 GTK_MESSAGE_QUESTION))
    uc_project_save ();
  else if (uc_project_get_save_bookmarks ())
    uc_application_save_bookmarks ();

  WINIT ("open_project_dialog", glade_open_project_dialog,
	 open_project_dialog);

  if (treestore_projects != NULL)
    g_object_unref (treestore_projects), treestore_projects = NULL;

  treestore_projects = gtk_list_store_new (N_COLUMNS, G_TYPE_INT,
					   GDK_TYPE_PIXBUF, G_TYPE_STRING);

  list = g_list_first (uc_project_get_projects_list ());
  while (list != NULL)
    {
      UCProjectProjects *item = (UCProjectProjects *) list->data;
      GdkPixbuf *icon = NULL;

      list = g_list_next (list);

      icon = uc_project_get_type_icon (item->type);
      gtk_list_store_append (treestore_projects, &iter);
      gtk_list_store_set (treestore_projects, &iter,
			  UID_COLUMN, item->id,
			  ICON_COLUMN, icon, TITLE_COLUMN, item->title, -1);
      g_object_unref (icon), icon = NULL;
    }

  WGET ("treeview", glade_open_project_dialog, treeview);

  gtk_tree_view_set_model (GTK_TREE_VIEW (treeview),
			   (GtkTreeModel *) treestore_projects);

  trenderer = gtk_cell_renderer_text_new ();
  prenderer = gtk_cell_renderer_pixbuf_new ();

  gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (treeview), -1,
					       _("Id"), trenderer,
					       "text", UID_COLUMN, NULL);
  column = gtk_tree_view_get_column (GTK_TREE_VIEW (treeview), UID_COLUMN);
  gtk_tree_view_column_set_visible (column, FALSE);

  gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (treeview), -1,
					       _("Type"), prenderer,
					       "pixbuf", ICON_COLUMN, NULL);

  gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW
					       (treeview), -1,
					       _("URL"), trenderer, "text",
					       TITLE_COLUMN, NULL);
  column = gtk_tree_view_get_column (GTK_TREE_VIEW (treeview), TITLE_COLUMN);
  gtk_tree_view_column_set_sort_column_id (column, TITLE_COLUMN);

  gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (treeview), TRUE);

  gtk_widget_show_all (open_project_dialog);
}

/**
 * uc_application_project_information_dialog_show:
 * 
 * Show the project properties dialog.
 *
 * Returns: TRUE if project must be saved.
 */
gboolean
uc_application_project_information_dialog_show (void)
{
  GtkWidget *widget = NULL;
  GladeXML *g = NULL;
  gboolean ret = FALSE;
  gint res = 0;

  WINIT ("project_information_dialog",
	 glade_project_information_dialog, project_information_dialog);
  g = glade_project_information_dialog;

  WGET ("description", g, widget);
  widget = (GtkWidget *) gtk_text_view_get_buffer (GTK_TEXT_VIEW (widget));
  gtk_text_buffer_set_text (GTK_TEXT_BUFFER (widget),
			    uc_project_get_description (), -1);
  WGET ("title", g, widget);
  gtk_entry_set_text (GTK_ENTRY (widget), uc_project_get_title ());

again:

  res = gtk_dialog_run (GTK_DIALOG (project_information_dialog));
  if (res == GTK_RESPONSE_OK)
    {
      if (!uc_project_save_properties ())
	goto again;
      ret = TRUE;
    }
  else
    gtk_widget_destroy (project_information_dialog),
      project_information_dialog = NULL;

  return ret;
}

static void
uc_application_auth_warning_dialog_not_display_cb (void)
{
  uc_project_set_prompt_auth (FALSE);
}

static void
uc_application_cookie_warning_dialog_not_display_cb (void)
{
  uc_project_set_cookies_warn_added (FALSE);
  uc_project_set_cookies_warn_updated (FALSE);
  uc_project_set_cookies_warn_deleted (FALSE);
}

/**
 * uc_application_cookie_warning_dialog_show:
 * @label: dialog's label.
 * @name: cookie's name.
 * @value: cookie's value.
 * @path: cookie's path.
 * @action: action to perform.
 *
 * Dialog for accepting/refusing a cookie.
 *
 * Returns: TRUE if the cookie must be accepted.
 *
 */
gboolean
uc_application_cookie_warning_dialog_show (const gchar * label,
					   gchar ** name,
					   gchar ** value,
					   gchar ** path,
					   gchar ** expires,
					   const UCCookiesAction action)
{
  GtkWidget *widget = NULL;
  GtkWidget *w = NULL;
  GladeXML *g = NULL;
  gint ret = 0;

  WINIT ("cookie_warning_dialog", g, w);
  WGET ("default_focus", g, widget);
  gtk_window_set_focus (GTK_WINDOW (w), widget);

  WGET ("stop_display", g, widget);
  g_signal_connect (G_OBJECT (widget), "toggled",
		    G_CALLBACK
		    (uc_application_cookie_warning_dialog_not_display_cb),
		    NULL);

  WGET ("label", g, widget);
  gtk_label_set_markup (GTK_LABEL (widget), label);
  WGET ("expires", g, widget);
  gtk_entry_set_text (GTK_ENTRY (widget), (*expires) ?
		      *expires : _("At the end of this session"));
  WGET ("name", g, widget);
  gtk_entry_set_text (GTK_ENTRY (widget), *name);
  WGET ("value", g, widget);
  gtk_entry_set_text (GTK_ENTRY (widget), *value);
  WGET ("path", g, widget);
  gtk_entry_set_text (GTK_ENTRY (widget), *path);

  if (action == UC_COOKIES_ACTION_UPDATE)
    {
      WSENS (g, "name", FALSE);
      WSENS (g, "path", FALSE);
    }
  else if (action == UC_COOKIES_ACTION_DELETE)
    {
      WSENS (g, "name", FALSE);
      WSENS (g, "value", FALSE);
      WSENS (g, "path", FALSE);
      WSENS (g, "expires", FALSE);
    }

  ret = gtk_dialog_run (GTK_DIALOG (w));

  /* cancel check */
  if (ret == GTK_RESPONSE_CANCEL)
    uc_check_cancel_set_value (TRUE);

  if (ret == GTK_RESPONSE_ACCEPT && action != UC_COOKIES_ACTION_DELETE)
    {
      g_free (*name), *name = NULL;
      WGET ("name", g, widget);
      *name = g_strdup (gtk_entry_get_text (GTK_ENTRY (widget)));
      g_free (*value), *value = NULL;
      WGET ("value", g, widget);
      *value = g_strdup (gtk_entry_get_text (GTK_ENTRY (widget)));
      g_free (*path), *path = NULL;
      WGET ("path", g, widget);
      *path = g_strdup (gtk_entry_get_text (GTK_ENTRY (widget)));
      WGET ("expires", g, widget);
      if (strcmp (gtk_entry_get_text (GTK_ENTRY (widget)),
		  _("At the end of this session")) != 0)
	{
	  g_free (*expires), *expires = NULL;
	  *expires = g_strdup (gtk_entry_get_text (GTK_ENTRY (widget)));
	}
    }

  gtk_widget_destroy (w), w = NULL;

  return (ret == GTK_RESPONSE_ACCEPT);
}

/**
 * uc_application_auth_dialog_show:
 * @title: string returned by the remote host when
 *         requested authentication.
 *
 * Show the basic authentication dialog.
 *
 * Returns: TRUE if both user and password have been
 *          entered.
 *
 */
gboolean
uc_application_auth_dialog_show (const gchar * title)
{
  GtkWidget *widget = NULL;
  GtkWidget *w = NULL;
  GladeXML *g = NULL;
  gchar *tmp = NULL;
  gboolean ret = FALSE;

  WINIT ("auth_dialog", g, w);
  if (title)
    {
      gchar **real_title = g_strsplit (title, "=", 2);
      tmp = g_strdup_printf (_("Enter login and password for %s at %s"),
			     real_title[1], uc_project_get_url ());

      WGET ("title", g, widget);
      gtk_label_set_text (GTK_LABEL (widget), tmp);

      g_free (tmp), tmp = NULL;
      g_strfreev (real_title), real_title = NULL;
    }

  WGET ("stop_display", g, widget);
  g_signal_connect (G_OBJECT (widget), "toggled",
		    G_CALLBACK
		    (uc_application_auth_warning_dialog_not_display_cb),
		    NULL);

again:

  if (gtk_dialog_run (GTK_DIALOG (w)) == GTK_RESPONSE_OK)
    {
      gchar *auth_user = NULL;
      gchar *auth_password = NULL;
      gchar *auth_line = NULL;

      if (uc_utils_get_auth_fields (g,
				    &auth_user, &auth_password, &auth_line))
	{
	  uc_project_set_auth_user (auth_user);
	  uc_project_set_auth_password (auth_password);
	  uc_project_set_auth_line (auth_line);
	  g_free (auth_line), auth_line = NULL;
	  ret = TRUE;
	}
      else
	goto again;
    }

  gtk_widget_destroy (w), w = NULL;

  return ret;
}

/**
 * uc_application_new_search_dialog_show:
 * 
 * Show the search dialog.
 */
void
uc_application_new_search_dialog_show (void)
{
  GladeXML *g = NULL;

  WINIT ("new_search_dialog", glade_new_search_dialog, new_search_dialog);
  g = glade_new_search_dialog;

  WSENS (g, "search_links_name", FALSE);
  WSENS (g, "search_meta_content", FALSE);
  WSENS (g, "search_emails", FALSE);

  gtk_widget_show_all (new_search_dialog);
}

/**
 * uc_application_search_dialog_show:
 * 
 * Show the search dialog.
 */
void
uc_application_search_dialog_show (void)
{
  WINIT ("search_dialog", glade_search_dialog, search_dialog);

  gtk_widget_show_all (search_dialog);
}

/**
 * uc_application_add_filter_domain_dialog_show:
 * 
 * Show the add domain filter dialog.
 */
void
uc_application_add_filter_domain_dialog_show (void)
{
  WINIT ("add_filter_domain_dialog", glade_add_filter_domain_dialog,
	 add_filter_domain_dialog);

  gtk_widget_show_all (add_filter_domain_dialog);
}

/**
 * uc_application_add_filter_directory_dialog_show:
 * 
 * Show the add directory filter dialog.
 */
void
uc_application_add_filter_directory_dialog_show (void)
{

  WINIT ("add_filter_directory_dialog",
	 glade_add_filter_directory_dialog, add_filter_directory_dialog);

  gtk_widget_show_all (add_filter_directory_dialog);
}

/**
 * uc_application_page_information_dialog_show:
 * 
 * Show the properties dialog.
 */
void
uc_application_page_information_dialog_show (void)
{
  gint32 id = 0;
  UCLinkProperties *prop = NULL;
  GtkWidget *widget = NULL;
  GladeXML *g = NULL;

  id = uc_check_treeview_get_selected_row_id ();
  if (id <= 0)
    return;

  prop = uc_lists_checked_links_lookup_by_uid (id);

  /* do not display link information for bookmark's
   * folders */
  if (prop->link_type == LINK_TYPE_BOOKMARK_FOLDER)
    return;

  WINIT ("page_information_dialog", glade_page_information_dialog,
	 page_information_dialog);
  g = glade_page_information_dialog;

  uc_application_page_information_dialog_display_header (prop);

  if (prop->metas != NULL)
    uc_application_page_information_dialog_display_meta_tags (prop);
  else
    {
      WGET ("meta_label", g, widget);
      gtk_widget_hide (widget);

      WGET ("meta_box", g, widget);
      gtk_widget_hide (widget);
    }

  WGET ("information_notebook", g, widget);
  gtk_notebook_set_current_page (GTK_NOTEBOOK (widget), 0);

  if (prop->link_type == LINK_TYPE_IMAGE ||
      prop->link_type == LINK_TYPE_EMAIL || prop->all_links == NULL)
    {
      gtk_notebook_remove_page (GTK_NOTEBOOK (widget), 2);
      gtk_notebook_remove_page (GTK_NOTEBOOK (widget), 1);
    }
  else
    {
      uc_application_page_information_dialog_display_links (prop);

      if (prop->emails != NULL)
	uc_application_page_information_dialog_display_emails (prop);
      else
	gtk_notebook_remove_page (GTK_NOTEBOOK (widget), 2);
    }

  gtk_widget_show (page_information_dialog);
}

/**
 * uc_application_display_search_message:
 * @label_pos: Position of the given message.
 * @message: Message to display.
 * 
 * Display messages in the progess dialog during
 * the search.
 */
void
uc_application_display_search_message (const guint label_pos,
				       const gchar * message)
{
  GtkWidget *label = NULL;
  gchar *tmp = NULL;

  if (uc_search_get_exit () || message == NULL ||
      search_dialog == NULL || glade_search_dialog == NULL ||
      !GTK_IS_WIDGET (search_dialog))
    return;

  tmp = (label_pos != UC_CHECK_MESSAGE_LABEL_FIRST) ?
    uc_utils_string_format4display (message, UC_LABEL_DISPLAY_MAX_LEN) :
    g_strdup (message);

  switch (label_pos)
    {
    case UC_CHECK_MESSAGE_LABEL_FIRST:
      WGET ("search_label_first", glade_search_dialog, label);
      break;

    case UC_CHECK_MESSAGE_LABEL_SECOND:
      WGET ("search_label_second", glade_search_dialog, label);
      break;

    default:
      g_free (tmp), tmp = NULL;
      g_return_if_reached ();
    }

  if (label_pos == UC_CHECK_MESSAGE_LABEL_FIRST)
    gtk_label_set_markup (GTK_LABEL (label), tmp);
  else
    gtk_label_set (GTK_LABEL (label), tmp);

  g_free (tmp), tmp = NULL;
}

/**
 * uc_application_display_state_message:
 * @label_pos: Position of the given message.
 * @message: Message to display.
 * 
 * Display messages in the progess dialog during
 * the check process
 */
void
uc_application_display_state_message (const guint label_pos,
				      const gchar * message)
{
  GtkWidget *label = NULL;
  gchar *tmp = NULL;

  g_return_if_fail (message || progress_dialog ||
		    GTK_IS_WIDGET (progress_dialog));

  tmp = (label_pos != UC_CHECK_MESSAGE_LABEL_THIRD) ?
    uc_utils_string_format4display (message, UC_LABEL_DISPLAY_MAX_LEN) :
    g_strdup (message);

  switch (label_pos)
    {
    case UC_CHECK_MESSAGE_LABEL_FIRST:
      WGET ("first_label", glade_progress_dialog, label);
      break;

    case UC_CHECK_MESSAGE_LABEL_SECOND:
      WGET ("second_label", glade_progress_dialog, label);
      break;

    case UC_CHECK_MESSAGE_LABEL_THIRD:
      WGET ("third_label", glade_progress_dialog, label);
      break;

    default:
      g_free (tmp), tmp = NULL;
      g_return_if_reached ();
    }

  if (label_pos == UC_CHECK_MESSAGE_LABEL_THIRD)
    gtk_label_set_markup (GTK_LABEL (label), tmp);
  else
    gtk_label_set (GTK_LABEL (label), tmp);

  g_free (tmp), tmp = NULL;
}

/**
 * uc_application_display_informations:
 * @treeview: Tree view to work with.
 * 
 * Search the header of a given url
 * in the urls list to display it.
 */
void
uc_application_display_informations (GtkTreeView * treeview)
{
  gint32 id = 0;
  UCLinkProperties *prop = NULL;
  UCStatusCode *sc = NULL;
  gchar *tmp_str = NULL;
  GtkTextBuffer *buffer = NULL;
  GtkWidget *text_informations = NULL;
  GtkWidget *location = NULL;
  GtkWidget *referrer = NULL;

  id = uc_check_treeview_get_selected_row_id ();
  if (id < 0)
    {
      uc_application_dialog_show (_
				  ("Please, select a item first."),
				  GTK_MESSAGE_WARNING);

      return;
    }

  prop = uc_lists_checked_links_lookup_by_uid (id);

  WGET ("text_informations", glade_main_window, text_informations);
  WGET ("location", glade_main_window, location);
  WGET ("referrer", glade_main_window, referrer);

  /* empty the buffer if the selected item is a bookmark's
   * folder */
  buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (text_informations));
  if (prop->link_type != LINK_TYPE_BOOKMARK_FOLDER)
    {
      sc =
	uc_application_get_status_code_properties (g_hash_table_lookup
						   (prop->header,
						    UC_HEADER_STATUS));

      tmp_str = g_strconcat ("<b><i>", sc->message, "</i></b>", NULL);
      gtk_label_set_markup (GTK_LABEL
			    (glade_xml_get_widget
			     (glade_main_window,
			      "label_informations_message")), tmp_str);
      g_free (tmp_str), tmp_str = NULL;

      gtk_entry_set_text (GTK_ENTRY (location), prop->url);
      gtk_entry_set_text (GTK_ENTRY (referrer),
			  (prop->parent && prop->parent->url) ?
			  prop->parent->url : _("No referrer"));

      gtk_text_buffer_set_text (buffer, sc->description, -1);
    }
  else
    {
      gtk_label_set (GTK_LABEL
		     (glade_xml_get_widget
		      (glade_main_window, "label_informations_message")),
		     " ");
      gtk_entry_set_text (GTK_ENTRY (location), " ");
      gtk_entry_set_text (GTK_ENTRY (referrer), " ");
      gtk_text_buffer_set_text (buffer, " ", -1);
    }
}

/**
 * uc_application_treeview_activate_popup:
 * @event: Event.
 * 
 * Display the treview popup menu.
 */
void
uc_application_treeview_activate_popup (GdkEventButton * event)
{
  GladeXML *xml = NULL;
  GtkWidget *popup = NULL;
  gchar *status_code = NULL;
  gint32 id = 0;
  UCLinkProperties *prop = NULL;

  id = uc_check_treeview_get_selected_row_id ();
  if (id < 0)
    return;

  prop = uc_lists_checked_links_lookup_by_uid (id);

  /* do not display information for bookmark's
   * folders */
  if (prop->link_type == LINK_TYPE_BOOKMARK_FOLDER)
    return;

  status_code = g_hash_table_lookup (prop->header, UC_HEADER_STATUS);

  WINIT ("treeview_popup_menu", xml, popup);

  gtk_menu_popup (GTK_MENU (popup), NULL, NULL, NULL, NULL,
		  event->button, event->time);

  /* activate/eactivate appropriate items */
  WSENS (xml, "refresh_branch", FALSE);
  WSENS (xml, "delete_link", FALSE);

  if (prop->depth_level == 0)
    WSENS (xml, "refresh_parent", FALSE);

  if (prop->link_type != LINK_TYPE_EMAIL)
    WSENS (xml, "menu_check_email_mx", FALSE);
  else
    {
      WSENS (xml, "view_online", FALSE);
      WSENS (xml, "refresh_link", FALSE);
    }

  if (!uc_check_status_is_good (status_code) ||
      !prop->is_downloadable ||
      (prop->link_type != LINK_TYPE_HREF &&
       prop->link_type != LINK_TYPE_FRAME
       && prop->link_type != LINK_TYPE_CSS))
    WSENS (xml, "view_link_content", FALSE);

  if (uc_project_get_speed_check () ||
      !uc_project_get_download_images_content () ||
      (prop->link_type != LINK_TYPE_IMAGE ||
       !uc_check_status_is_good (status_code)))
    WSENS (xml, "image_preview", FALSE);

  if (uc_project_get_bookmarks_file () != NULL)
    {
      WSENS (xml, "refresh_parent", FALSE);
      WSENS (xml, "view_similar_link_locations", FALSE);
      WSENS (xml, "view_parent_page_online", FALSE);
      WSENS (xml, "delete_link", TRUE);
    }

  if (prop->childs != NULL)
    WSENS (xml, "refresh_branch", TRUE);

  if (uc_check_status_is_malformed (status_code))
    {
      WSENS (xml, "view_online", FALSE);
      WSENS (xml, "refresh_link", FALSE);
    }

  gtk_widget_show_all (popup);
}

/**
 * uc_application_buttons_save_setting:
 * @widget: widget to work with. 
 * @path: "documents" or "images".
 * 
 * Saving states of a notebook check
 * button.
 */
static void
uc_application_buttons_save_setting (GtkWidget * widget, gchar * path)
{
  GList *items = NULL;
  gchar **radios = NULL;
  guint size = 0;
  GString *str = g_string_new ("");

  items = g_list_first (uc_application_buttons_get_list (widget));
  while (items != NULL)
    {
      if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (items->data)))
	{
	  g_string_append_printf (str, ",%s",
				  gtk_button_get_label
				  (GTK_BUTTON (items->data)));
	  size++;
	}

      items = items->next;
    }
  g_list_free (items), items = NULL;

  if (size > 0)
    g_string_erase (str, 0, 1);

  radios = g_strsplit (str->str, ",", UC_CHECK_BUTTON_MAX_DISPLAY);

  if (!strcmp (path, "documents"))
    uc_project_set_reject_documents (str->str);
  else if (!strcmp (path, "images"))
    uc_project_set_reject_images (str->str);

  g_strfreev (radios), radios = NULL;
  g_string_free (str, TRUE), str = NULL;
}

/**
 * uc_application_buttons_get_list:
 * @widget: 
 * 
 * FIXME
 * Return a GList of pointers to the check buttons
 * of the notebooks
 *
 * Returns: #GList of references n check buttons.
 */
static GList *
uc_application_buttons_get_list (const GtkWidget * widget)
{
  GList *parents = NULL;
  GList *childs = NULL;
  GList *list = NULL;

  parents =
    g_list_first (gtk_container_get_children (GTK_CONTAINER (widget)));
  while (parents != NULL)
    {
      if (GTK_IS_TABLE (parents->data))
	{
	  childs =
	    g_list_first (gtk_container_get_children
			  (GTK_CONTAINER (parents->data)));
	  while (childs != NULL)
	    {
	      list = g_list_append (list, childs->data);
	      childs = g_list_next (childs);
	    }
	  g_list_free (childs), childs = NULL;
	}
      parents = g_list_next (parents);
    }

  g_list_free (parents), parents = NULL;

  return list;
}

/**
 * uc_application_buttons_display:
 * @frame: Frame where displaying buttons.
 * @array: Reference on array of buttons to display.
 * @items_size: Buttons array size.
 * @selected_array: Selected buttons on array.
 * @selected_items_size: Size of buttons selected aray.
 * 
 * Dislay items in a frame.
 */
static void
uc_application_buttons_display (const GtkWidget * frame, gpointer array,
				const guint items_size,
				gpointer selected_array,
				const guint selected_items_size)
{
  GtkWidget *checkButton = NULL;
  GtkWidget *table = NULL;
  gchar **items = (gchar **) array;
  gchar **selected_items = (gchar **) selected_array;
  guint i = 0;
  guint col = 0;
  guint row = 0;
  gdouble maxRows = 0;
  gdouble maxCols = UC_DIALOG_SETTINGS_DOCUMENT_TYPES_MAX_COLS;

  maxRows = ceil (items_size / maxCols);

  table = gtk_table_new (maxCols, maxRows, FALSE);
  gtk_widget_show_all (table);
  gtk_container_add (GTK_CONTAINER (frame), table);
  gtk_container_set_border_width (GTK_CONTAINER (table), 5);
  gtk_table_set_row_spacings (GTK_TABLE (table), 2);
  gtk_table_set_col_spacings (GTK_TABLE (table), 2);

  for (row = 0, i = 0; row < maxRows; row++, col = 0)
    while (col < maxCols && i < items_size)
      {
	checkButton = gtk_check_button_new_with_mnemonic (items[i]);

	if (uc_utils_array_is_in
	    (items[i], selected_items, selected_items_size))
	  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (checkButton),
					TRUE);

	gtk_widget_show_all (checkButton);
	gtk_table_attach (GTK_TABLE (table), checkButton,
			  col, col + 1, row, row + 1,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);
	col++, i++;
      }
}

/**
 * uc_application_remove_paths:
 * 
 * Remove all application paths
 */
void
uc_application_remove_paths (void)
{
  gchar *final_path = NULL;

  final_path =
    g_strconcat (uc_project_get_working_path (), "/",
		 uc_project_get_cache_name (), NULL);

  uc_utils_rmdirs (final_path, FALSE);

  g_free (final_path), final_path = NULL;
}

/**
 * uc_application_settings_dialog_display_filters_list:
 * @glade: the appropriate glade pointer.
 * @elements: a vector of items to display.
 * @items_size: size of the vector.
 * @tv: tree view to display to work with.
 * @model: list store to work with.
 * 
 * Display list of elements to reject.
 */
static void
uc_application_settings_dialog_display_filters_list (const gchar * glade,
						     gpointer elements,
						     guint items_size,
						     GtkTreeView ** tv,
						     GtkListStore ** model)
{
  enum
  {
    NAME_COLUMN,
    N_COLUMNS
  };

  gchar **items = NULL;
  GtkWidget *scroll = NULL;
  GtkTreeIter iter;
  GtkCellRenderer *renderer;
  guint i = 0;

  WGET (glade, glade_settings_dialog, scroll);

  if (*model != NULL)
    g_object_unref (*model), *model = NULL;

  *model = gtk_list_store_new (N_COLUMNS, G_TYPE_STRING);

  if (elements != NULL)
    {
      items = (gchar **) elements;

      for (i = 0; i < items_size; i++)
	{
	  gtk_list_store_append (*model, &iter);
	  gtk_list_store_set (*model, &iter, NAME_COLUMN, (gchar *) items[i],
			      -1);
	}
    }

  *tv =
    (GtkTreeView *) gtk_tree_view_new_with_model ((GtkTreeModel *) * model);

  renderer = gtk_cell_renderer_text_new ();
  gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (*tv),
					       -1,
					       _("Name"), renderer,
					       "text", NAME_COLUMN, NULL);
  uc_utils_set_userfriendly_treeview_column (GTK_TREE_VIEW (*tv),
					     NAME_COLUMN);

  gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (*tv), TRUE);

  gtk_container_add (GTK_CONTAINER (scroll), (GtkWidget *) * tv);

  gtk_widget_show_all (scroll);
}

/**
 * uc_application_page_information_dialog_display_header:
 * @prop: #UCLinkProperties node of the element to work with.
 * 
 * Display header information.
 */
static void
uc_application_page_information_dialog_display_header (const
						       UCLinkProperties *
						       prop)
{
  GtkWidget *widget = NULL;
  gchar *tmp = NULL;
  guint status_tmp = 0;

  /* URL */
  WGET ("location", glade_page_information_dialog, widget);
  tmp = (strlen (prop->url) > 0) ?
    g_strdup (prop->url) : g_strdup (_("Missing URL"));
  gtk_entry_set_text (GTK_ENTRY (widget), tmp);
  g_free (tmp), tmp = NULL;

  /* Link type */
  WGET ("link_type", glade_page_information_dialog, widget);
  tmp = g_strdup_printf ("(%s)",
			 uc_check_get_link_type_label (prop->link_type));
  gtk_label_set (GTK_LABEL (widget), tmp);
  g_free (tmp), tmp = NULL;
  WGET ("link_type_image", glade_page_information_dialog, widget);
  gtk_image_set_from_file (GTK_IMAGE (widget),
			   uc_check_get_link_type_icon_path (prop->link_type,
							     prop->protocol));

  /* referring URL */
  WGET ("referrer", glade_page_information_dialog, widget);

  /* there is no parent url for a local file or bookmark */
  if (uc_project_get_bookmarks_file () == NULL)
    tmp = (prop->parent && prop->parent->url) ?
      g_strdup (prop->parent->url) : g_strdup (_("No referrer"));
  else
    tmp = g_strdup (_("No referrer"));
  gtk_entry_set_text (GTK_ENTRY (widget), tmp);
  g_free (tmp), tmp = NULL;

  /* server type */
  WGET ("server_type", glade_page_information_dialog, widget);
  tmp = g_hash_table_lookup (prop->header, UC_SERVER);
  tmp = (tmp == NULL) ?
    g_strdup (_("Not specified")) :
    uc_utils_get_server_from_header_field (tmp);
  gtk_label_set (GTK_LABEL (widget), tmp);
  g_free (tmp), tmp = NULL;

  /* status */
  WGET ("status_image", glade_page_information_dialog, widget);
  gtk_image_set_from_pixbuf (GTK_IMAGE (widget), prop->status_icon);
  WGET ("status", glade_page_information_dialog, widget);
  if ((tmp = g_hash_table_lookup (prop->header, UC_HEADER_STATUS)) == NULL)
    tmp = g_strdup_printf ("(%s)", _("Unknown"));
  else
    {
      status_tmp = atoi (tmp);
      if (status_tmp >= UC_STATUS_CODE_INTERNALS_LIMIT)
	tmp = g_strdup (_("(gURLChecker internals)"));
      else
	tmp = g_strdup_printf ("(%s)", tmp);
    }
  gtk_label_set (GTK_LABEL (widget), tmp);
  g_free (tmp), tmp = NULL;

  /* cached */
  WGET ("source", glade_page_information_dialog, widget);
  gtk_label_set (GTK_LABEL (widget), (prop->is_downloadable) ?
		 _("Disk Cache") : _("Not cached"));
  g_free (tmp), tmp = NULL;

  /* content length */
  WGET ("size", glade_page_information_dialog, widget);
  tmp = g_hash_table_lookup (prop->header, UC_CONTENT_LENGTH);
  if (tmp == NULL)
    tmp = g_strdup (_("Not specified"));
  else
    tmp = uc_utils_get_string_from_size (atol (tmp));
  gtk_label_set (GTK_LABEL (widget), tmp);
  g_free (tmp), tmp = NULL;

  /* last modified */
  WGET ("modified", glade_page_information_dialog, widget);
  tmp = g_strdup (g_hash_table_lookup (prop->header, UC_LAST_MODIFIED));
  if (tmp == NULL)
    tmp = g_strdup (_("Not specified"));
  gtk_label_set (GTK_LABEL (widget), tmp);
  g_free (tmp), tmp = NULL;
}

/**
 * uc_application_page_information_dialog_display_emails:
 * @prop: #UCLinkProperties node of the element to work with.
 * 
 * Display page information emails list.
 */
static void
uc_application_page_information_dialog_display_emails (const
						       UCLinkProperties *
						       prop)
{
  enum
  {
    LABEL_COLUMN,
    URL_COLUMN,
    N_COLUMNS
  };
  GtkTreeView *tv;
  GtkTreeIter iter;
  GtkListStore *model;
  GtkCellRenderer *renderer;
  GList *item = NULL;
  GtkWidget *scroll = NULL;

  WGET ("emails_scrolled", glade_page_information_dialog, scroll);

  model = gtk_list_store_new (N_COLUMNS, G_TYPE_STRING, G_TYPE_STRING);

  item = g_list_first (prop->emails);
  while (item != NULL)
    {
      UCHTMLTag *link = (UCHTMLTag *) item->data;

      gtk_list_store_append (model, &iter);
      gtk_list_store_set (model, &iter,
			  URL_COLUMN, link->value,
			  LABEL_COLUMN, link->label, -1);

      item = g_list_next (item);
    }

  tv = (GtkTreeView *) gtk_tree_view_new_with_model ((GtkTreeModel *) model);
  g_object_unref (model), model = NULL;

  renderer = gtk_cell_renderer_text_new ();
  gtk_tree_view_insert_column_with_attributes (tv,
					       -1,
					       _("Email"), renderer,
					       "text", URL_COLUMN, NULL);
  uc_utils_set_userfriendly_treeview_column (tv, URL_COLUMN);

  gtk_tree_view_insert_column_with_attributes (tv,
					       -1,
					       _("Label"), renderer,
					       "text", LABEL_COLUMN, NULL);
  uc_utils_set_userfriendly_treeview_column (tv, LABEL_COLUMN);

  gtk_tree_view_set_rules_hint (tv, TRUE);

  gtk_container_add (GTK_CONTAINER (scroll), GTK_WIDGET (tv));

  gtk_widget_show_all (scroll);
}

/**
 * uc_application_page_information_dialog_display_links:
 * @prop: #UCLinkProperties node of the element to work with.
 * 
 * Display page information urls list.
 */
static void
uc_application_page_information_dialog_display_links (const UCLinkProperties
						      * prop)
{
  enum
  {
    LABEL_COLUMN,
    URL_COLUMN,
    N_COLUMNS
  };
  GtkTreeView *tv;
  GtkTreeIter iter;
  GtkListStore *model;
  GtkCellRenderer *renderer;
  GList *item = NULL;
  GtkWidget *scroll = NULL;
  gchar *label = NULL;
  gchar *value = NULL;

  WGET ("links_scrolled", glade_page_information_dialog, scroll);

  model = gtk_list_store_new (N_COLUMNS, G_TYPE_STRING, G_TYPE_STRING);

  item = g_list_first (prop->all_links);
  while (item != NULL)
    {
      UCHTMLTag *link = (UCHTMLTag *) item->data;

      label =
	uc_utils_string_format4display (link->label,
					UC_LABEL_DISPLAY_MAX_LEN);
      value =
	uc_utils_string_format4display (link->value,
					UC_LABEL_DISPLAY_MAX_LEN);

      gtk_list_store_append (model, &iter);
      gtk_list_store_set (model, &iter,
			  URL_COLUMN, value, LABEL_COLUMN, label, -1);

      g_free (value), value = NULL;
      g_free (label), label = NULL;

      item = g_list_next (item);
    }

  tv = (GtkTreeView *) gtk_tree_view_new_with_model ((GtkTreeModel *) model);
  g_object_unref (model), model = NULL;

  renderer = gtk_cell_renderer_text_new ();
  gtk_tree_view_insert_column_with_attributes (tv,
					       -1,
					       _("URL"), renderer,
					       "text", URL_COLUMN, NULL);
  uc_utils_set_userfriendly_treeview_column (tv, URL_COLUMN);

  gtk_tree_view_insert_column_with_attributes (tv,
					       -1,
					       _("Label"), renderer,
					       "text", LABEL_COLUMN, NULL);
  uc_utils_set_userfriendly_treeview_column (tv, LABEL_COLUMN);

  gtk_tree_view_set_rules_hint (tv, TRUE);

  gtk_container_add (GTK_CONTAINER (scroll), GTK_WIDGET (tv));

  gtk_widget_show_all (scroll);
}

/**
 * uc_application_view_similar_links_dialog_show:
 * 
 * Show similar links dialog.
 */
void
uc_application_view_similar_links_dialog_show (void)
{
  GtkTreeView *tv;
  GtkTreeIter iter;
  GtkListStore *model;
  GtkCellRenderer *renderer;
  GList *item = NULL;
  UCLinkProperties *prop = NULL;
  GtkWidget *scrolled_similar_links = NULL;
  gint32 id = 0;

  id = uc_check_treeview_get_selected_row_id ();
  if (id < 0)
    {
      uc_application_dialog_show (_
				  ("Please, select a item first."),
				  GTK_MESSAGE_WARNING);
      return;
    }

  WINIT ("similar_links_dialog", glade_similar_links_dialog,
	 similar_links_dialog);
  WGET ("scrolled_similar_links", glade_similar_links_dialog,
	scrolled_similar_links);

  model = gtk_list_store_new (1, G_TYPE_STRING);

  prop = uc_lists_checked_links_lookup_by_uid (id);

  item = g_list_first (prop->similar_links_parents);
  while (item != NULL)
    {
      gchar *str = (gchar *) item->data;

      item = g_list_next (item);

      gtk_list_store_append (model, &iter);
      gtk_list_store_set (model, &iter, 0, str, -1);
    }

  tv = (GtkTreeView *) gtk_tree_view_new_with_model ((GtkTreeModel *) model);
  g_object_unref (model), model = NULL;

  renderer = gtk_cell_renderer_text_new ();
  gtk_tree_view_insert_column_with_attributes (tv,
					       -1, _("URL"),
					       renderer, "text", 0, NULL);
  uc_utils_set_userfriendly_treeview_column (tv, 0);

  gtk_tree_view_set_rules_hint (tv, TRUE);

  gtk_container_add (GTK_CONTAINER (scrolled_similar_links), GTK_WIDGET (tv));

  gtk_widget_show_all (similar_links_dialog);
}

/**
 * uc_application_launch_web_browser:
 * @url: URL to pass to the web browser.
 * 
 * Launch a web browser to view the selected
 * url
 */
void
uc_application_launch_web_browser (const gchar * url)
{
  gchar *message = NULL;
  GError *err = NULL;
  gchar *command_line = NULL;
  gchar *browser = NULL;
  gint32 ret = 0;

  browser = uc_project_get_browser_path ();
  command_line = g_strconcat (browser, " \"", url, "\"", NULL);

  ret = g_spawn_command_line_async (command_line, &err);
  g_free (command_line), command_line = NULL;

  if (ret == 0)
    {
      message = g_strdup_printf ("%s\n\n%s",
				 _
				 ("Cannot start your web browser; "
				  "the following error occured:"),
				 err->message);

      uc_application_dialog_show (message, GTK_MESSAGE_ERROR);

      g_clear_error (&err);
      g_free (message), message = NULL;
    }
}

/**
 * uc_application_view_image_dialog_show:
 * 
 * Show view image dialog.
 */
void
uc_application_view_image_dialog_show (void)
{
  gint32 id = 0;
  gchar *src = NULL;

  id = uc_check_treeview_get_selected_row_id ();
  if (id < 0)
    {
      uc_application_dialog_show (_
				  ("Please, select a item first."),
				  GTK_MESSAGE_WARNING);

      return;
    }

  src = uc_utils_convert_uid2file (id);
  if (src != NULL)
    {
      GtkImage *image_preview = NULL;

      WINIT ("view_image_dialog", glade_view_image_dialog, view_image_dialog);

      image_preview =
	GTK_IMAGE (glade_xml_get_widget
		   (glade_view_image_dialog, "image_preview"));

      gtk_image_set_from_file (image_preview, src);

      g_free (src), src = NULL;

      gtk_widget_show_all (view_image_dialog);
    }
  else
    uc_application_dialog_show (_("The corresponding "
				  "file is empty or does not exist "
				  "in the cache directory!"),
				GTK_MESSAGE_ERROR);
}

/**
 * uc_application_view_source_dialog_show:
 * 
 * Show view source dialog.
 */
void
uc_application_view_source_dialog_show (void)
{
  gint32 id = 0;
  gchar *src = NULL;
  gsize length = 0;

  id = uc_check_treeview_get_selected_row_id ();
  if (id < 0)
    {
      uc_application_dialog_show (_
				  ("Please, select a item first."),
				  GTK_MESSAGE_WARNING);

      return;
    }

  src = uc_cache_get_source (id, &length);
  if (src != NULL)
    {
      gchar *tmp = NULL;
      GtkWidget *text_view_source = NULL;
      GtkTextBuffer *buffer = NULL;

      WINIT ("view_source_dialog", glade_view_source_dialog,
	     view_source_dialog);
      WGET ("text_view_source", glade_view_source_dialog, text_view_source);

      buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (text_view_source));
      tmp = uc_utils_to_utf8 (src);
      gtk_text_buffer_set_text (buffer, tmp, -1);

      g_free (tmp), tmp = NULL;
      g_free (src), src = NULL;

      gtk_widget_show_all (view_source_dialog);
    }
  else
    uc_application_dialog_show (_("The corresponding "
				  "file is empty or does not exist "
				  "in the cache directory!"),
				GTK_MESSAGE_ERROR);
}

/**
 * uc_application_status_code_properties_free:
 * 
 * Free the code properties hash table.
 */
static void
uc_application_status_code_properties_free (void)
{
  guint i = 0;
  GHashNode *node = NULL;
  GHashTable *hash = uc_application_status_code_properties_hash;

  g_return_if_fail (hash != NULL);

  for (i = 0; i < hash->size; i++)
    for (node = hash->nodes[i]; node; node = node->next)
      {
	UCStatusCode *sc = (UCStatusCode *) node->value;

	g_object_unref (sc->icon_file), sc->icon_file = NULL;
	g_free (sc), sc = NULL;
      }

  g_hash_table_destroy (uc_application_status_code_properties_hash);
  uc_application_status_code_properties_hash = NULL;
}

/**
 * uc_application_status_code_properties_init:
 * 
 * This function load the properties to associate
 * with a server status code.
 */
void
uc_application_status_code_properties_init (void)
{
  guint i = 0;
  struct
  {
    gchar *code;
    gchar *color;
    gchar *icon;
    gchar *message;
    gchar *description;
  }
  UCHTMLCodes[] =
  {
    /* HOST UNREACHABLE - gURLChecker internal */
    {
    UC_STATUS_CODE_HOST_IS_UNREACHABLE,
	UC_TIMEOUT_LINK_BGCOLOR,
	"/link_status_timeout.png",
	_("Host unreachable"),
	_("The remote host does not respond at all...\nPerhaps "
	    "you would change the timeout in gURLChecker "
	    "preferences, or perhaps it is really down."
	    "\n\nTry to check it later.")},
      /* FILE PROTOCOL - gURLChecker internal */
    {
    UC_STATUS_CODE_FILE_PROTO_ERROR,
	UC_FILE_ERROR_LINK_BGCOLOR,
	"/link_status_file_error.png",
	_("FILE:// protocol"),
	_("File protocol detected.\n"
	    "You have asked me to warn you when I find the \"FILE://\" "
	    "protocol in a link.")},
      /* FTP OK - gURLChecker internal */
    {
    UC_STATUS_CODE_FTP_OK,
	UC_GOOD_LINK_BGCOLOR,
	"/link_status_good.png",
	_("Page available"),
	_("No problem with this page.\nServer is up and document "
	    "can be loaded without any problem.")},
      /* FTP BAD - gURLChecker internal */
    {
    UC_STATUS_CODE_FTP_BAD,
	UC_BAD_LINK_BGCOLOR,
	"/link_status_bad.png",
	_("Page not found"),
	_("The page does not longer exist on the server.")},
      /* BAD EMAIL LINK SYNTAX - gURLChecker internal */
    {
    UC_STATUS_CODE_BAD_EMAIL_LINK,
	UC_BAD_EMAIL_LINK_BGCOLOR,
	"/link_status_mail_bad.png",
	_("Bad email link"),
	_("The email address specified in the link is not well "
	    "formatted.")},
      /* BAD EMAIL LINK MX - gURLChecker internal */
    {
    UC_STATUS_CODE_BAD_EMAIL_LINK_MX,
	UC_BAD_EMAIL_LINK_BGCOLOR,
	"/link_status_mail_bad.png",
	_("Problem with MX"),
	_("The MX of the domain specified for this email address is "
	    "not valid or the domain itself is not valid.")},
      /* GOOD EMAIL LINK - gURLChecker internal */
    {
    UC_STATUS_CODE_GOOD_EMAIL_LINK,
	UC_GOOD_LINK_BGCOLOR,
	"/link_status_mail_good.png",
	_("Email link ok"), _("No problem with this email address.")},
      /* IGNORED ITEM - gURLChecker internal */
    {
    UC_STATUS_CODE_IGNORED_LINK,
	UC_IGNORED_LINK_BGCOLOR,
	"/link_status_ignored.png",
	_("Item ignored"),
	_("You chose to ignore this item while gURLChecker "
	    "was checking it.")},
      /* MALFORMED ITEM - gURLChecker internal */
    {
    UC_STATUS_CODE_MALFORMED_LINK,
	UC_MALFORMED_LINK_BGCOLOR,
	"/link_status_malformed.png",
	_("Malformed link"),
	_("The HTML link is malformed.\nPerhaps HREF value is missing?")},
      /* UNKNOW - gURLChecker internal */
    {
    UC_STATUS_CODE_UNKNOWN,
	UC_BAD_LINK_BGCOLOR,
	"/link_type_unknown.png",
	_("No valid information"),
	_("gURLChecker can't interpret the status code sent "
	    "by the remote host.\nPlease, report the value of the "
	    "\"Status\" header field to the author of gURLChecker.")},
      /* 200 */
    {
    UC_STATUS_CODE_PAGE_OK,
	UC_GOOD_LINK_BGCOLOR,
	"/link_status_good.png",
	_("Page available"),
	_("No problem with this page.\nServer is up and document "
	    "can be loaded without any problem.")},
      /* 301 */
    {
    UC_STATUS_CODE_MOVED_PERMANENTLY,
	UC_GOOD_LINK_BGCOLOR,
	"/link_status_good.png",
	_("Page moved permanently"),
	_("No real problem with this page.\n"
	    "The requested resource just has been assigned a new permanent "
	    "URI.")},
      /* 302 */
    {
    UC_STATUS_CODE_REDIRECTED,
	UC_GOOD_LINK_BGCOLOR,
	"/link_status_good.png",
	_("Page moved Temporarily"),
	_("No problem with this page.\nServer is up and document "
	    "can be loaded without any problem.")},
      /* 401 */
    {
    UC_STATUS_CODE_RESTRICTED,
	UC_RESTRICTED_LINK_BGCOLOR,
	"/link_status_restricted.png",
	_("Restricted access"),
	_("This page need authentication to be viewed.")},
      /* 403 */
    {
    "403",
	UC_BAD_LINK_BGCOLOR,
	"/link_status_bad.png",
	_("Access forbidden"),
	_("The remote host has refused to serve us the page.\n"
	    "Some web servers do not allow HEAD method on particular "
	    "files... and gURLChecker need to send HEAD method :-("
	    "\n\nTry to check it later, or filter this domain in the "
	    "settings.")},
      /* 204 */
    {
    UC_STATUS_CODE_NO_DATA,
	UC_NO_DATA_LINK_BGCOLOR,
	"/link_status_nodata.png",
	_("Page available, but with no content"),
	_("No problem with this page, but it have no content "
	    "at all...\n\nTry to check it later.")},
      /* 400 */
    {
    "400",
	UC_BAD_LINK_BGCOLOR,
	"/link_status_bad.png",
	_("Malformed URL"),
	_("The request could not be understood by the server "
	    "due to malformed syntax.")},
      /* 404 */
    {
    "404",
	UC_BAD_LINK_BGCOLOR,
	"/link_status_bad.png",
	_("Page not found"),
	_("The page does not longer exist on the server.")},
      /* 405 */
    {
    UC_STATUS_CODE_BAD_METHOD,
	UC_BAD_LINK_BGCOLOR,
	"/link_status_bad.png",
	_("Method not allowed"),
	_("The remote host has refused to serve us the page."
	    "\n\nTry to check it later.")},
      /* 500 */
    {
    "500",
	UC_BAD_LINK_BGCOLOR,
	"/link_status_bad.png",
	_("Internal server error"),
	_("The page contain bad code, or the server is "
	    "misconfigured.\n\nTry to check it later.")},
      /* 501 */
    {
    "501",
	UC_BAD_LINK_BGCOLOR,
	"/link_status_bad.png",
	_("Method not supported"),
	_("Some web servers do not allow HEAD method on particular "
	    "files... and gURLChecker need to send HEAD method :-("
	    "\n\nTry to check it later, or filter this domain in the "
	    "settings.")},
      /* 502 */
    {
    "502",
	UC_BAD_LINK_BGCOLOR,
	"/link_status_bad.png",
	_("Bad Gateway"),
	_("The server, while acting as a gateway or proxy, received an "
	    "invalid response from the upstream server it accessed in "
	    "attempting to fulfill the request.\n\nTry to check this "
	    "page later.")},
      /* 503 */
    {
    "503",
	UC_BAD_LINK_BGCOLOR,
	"/link_status_timeout.png",
	_("Service unavailable"),
	_("The server is currently unable to handle the request "
	    "due to a temporary overloading or maintenance "
	    "of the server. It is also possible that you are using "
	    "a proxy that can not join the given host.\n\nTry "
	    "to check it later, or change your proxy settings.")},
      /* 504 */
    {
    "504",
	UC_TIMEOUT_LINK_BGCOLOR,
	"/link_status_timeout.png",
	_("Request not completed"),
	_("The server, while acting as a gateway or proxy, did not "
	    "receive a timely response from the upstream server specified "
	    "by the URI (e.g. HTTP, FTP, LDAP) or some other auxiliary "
	    "server (e.g. DNS) it needed to access in attempting to "
	    "complete the request.")},
      /* FTP - 530 */
    {
    UC_STATUS_CODE_FTP_MAX_CLIENTS,
	UC_RESTRICTED_LINK_BGCOLOR,
	"/link_status_restricted.png",
	_("Access forbidden or too many connections"),
	_
	("This server do not accept anonymous users, or the maximum number "
	   "of allowed clients are already connected.\n\n"
	   "Try to check it later.")},
      /* 408 */
    {
  UC_STATUS_CODE_TIMEOUT,
	UC_TIMEOUT_LINK_BGCOLOR,
	"/link_status_timeout.png",
	_("Receiving timout requesting the page"),
	_("Perhaps you would change the timeout in gURLChecker "
	      "preferences. Perhaps the remote host is "
	      "too busy for the moment to responde to "
	      "gURLChecker request, or perhaps the remote host does not "
	      "exist at all.\n\nTry to check it later.")},};
  const guint codes_len = sizeof (UCHTMLCodes) / sizeof (UCHTMLCodes[0]);
  gchar *path = NULL;

  if (uc_application_status_code_properties_hash != NULL)
    uc_application_status_code_properties_free ();

  uc_application_status_code_properties_hash =
    g_hash_table_new (g_str_hash, g_str_equal);

  /* initialization of the hash table for HTTP
   * codes properties */
  for (i = 0; i < codes_len; i++)
    {
      UCStatusCode *sc = NULL;

      sc = g_new0 (UCStatusCode, 1);
      path = g_strconcat (UC_PIXMAPS_DIR, UCHTMLCodes[i].icon, NULL);
      sc->icon_file = gdk_pixbuf_new_from_file (path, NULL);
      g_free (path), path = NULL;
      sc->color = UCHTMLCodes[i].color;
      sc->message = UCHTMLCodes[i].message;
      sc->description = UCHTMLCodes[i].description;
      g_hash_table_insert (uc_application_status_code_properties_hash,
			   UCHTMLCodes[i].code, (gpointer) sc);
    }
}

/**
 * uc_application_new_instance_launch:
 * 
 * Launch a new instance of gURLChecker.
 */
void
uc_application_new_instance_launch (void)
{
  gchar *message = NULL;
  GError *err = NULL;
  gchar *argv[] = { PACKAGE_BIN_DIR "/" PACKAGE, NULL };
  gchar **envp = environ;
  GSpawnFlags flags = G_SPAWN_DO_NOT_REAP_CHILD;
  gint child_pid;

  if (g_spawn_async (NULL, argv, envp, flags, NULL, NULL, &child_pid, &err))
    return;

  message = g_strdup_printf ("%s\n\n%s",
			     _
			     ("Cannot start new instance of gURLChecker; "
			      "the following error occured:"), err->message);
  uc_application_dialog_show (message, GTK_MESSAGE_ERROR);
  g_clear_error (&err);

  g_free (message), message = NULL;
}


static void
uc_application_settings_cookies_toggled_cb (GtkToggleButton * button,
					    gpointer user_data)
{
  GladeXML *g = glade_settings_dialog;

  if (!strcmp (gtk_widget_get_name (GTK_WIDGET (button)),
	       "radio_cookies_accept"))
    WSENS (g, "frame_alerts", TRUE);
  else
    WSENS (g, "frame_alerts", FALSE);
}

/**
 * uc_application_settings_dialog_show:
 * 
 * Show the settings dialog box.
 */
void
uc_application_settings_dialog_show (void)
{
  GtkWidget *widget = NULL;
  gchar **items = NULL;
  gchar **selected_items = NULL;
  GtkWidget *frame = NULL;
  guint items_size = 0;
  guint selected_items_size = 0;
  GladeXML *g = NULL;

  WINIT ("settings_dialog", glade_settings_dialog, settings_dialog);
  g = glade_settings_dialog;

  /* network timeout */
  WGET ("timeout", g, widget);
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (widget),
			     (gfloat) uc_project_get_check_timeout ());

  /* prompt for auth */
  WGET ("prompt_auth", g, widget);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget),
				uc_project_get_prompt_auth ());
  /* debug mode? */
  WGET ("debug_mode", g, widget);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget),
				uc_project_get_debug_mode ());

  /* dump properties? */
  WGET ("dump_properties", g, widget);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget),
				uc_project_get_dump_properties ());

  /* networking depth level */
  WGET ("url_depth_level", g, widget);
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (widget),
			     (gfloat) uc_project_get_depth_level ());

  /* retrieve images content */
  WGET ("images_content", g, widget);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget),
				uc_project_get_download_images_content ());

  /* retrieve archives content */
  WGET ("archives_content", g, widget);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget),
				uc_project_get_download_archives_content ());

  /* networking check wait */
  WGET ("url_wait", g, widget);
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (widget),
			     (gfloat) uc_project_get_check_wait ());

  /* documents to reject */
  WGET ("frame_filters_documents", g, frame);
  items = UC_GET_DEFAULT_DOCUMENTS;
  items_size = UC_DEFAULT_DOCUMENTS_LEN;
  uc_project_get_reject_documents (&selected_items, &selected_items_size);
  uc_application_buttons_display ((GtkWidget *) frame, items,
				  items_size, selected_items,
				  selected_items_size);
  g_strfreev (items), items = NULL;

  /* images to reject */
  WGET ("frame_filters_images", g, frame);
  items = UC_GET_DEFAULT_IMAGES;
  items_size = UC_DEFAULT_IMAGES_LEN;
  uc_project_get_reject_images (&selected_items, &selected_items_size);
  uc_application_buttons_display (frame, items, items_size,
				  selected_items, selected_items_size);
  g_strfreev (items), items = NULL;

  /* directories to reject */
  uc_project_get_reject_directories (&items, &items_size);
  uc_application_settings_dialog_display_filters_list
    ("scrolled_filters_directories", items, items_size,
     &treeview_filter_directories, &treestore_filter_directories);

  /* domains to reject */
  uc_project_get_reject_domains (&items, &items_size);
  uc_application_settings_dialog_display_filters_list
    ("scrolled_filters_domains", items, items_size,
     &treeview_filter_domains, &treestore_filter_domains);

  /* value for domains in timeout */
  WGET ("timeouts_blocked", g, widget);
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (widget),
			     (gfloat) uc_project_get_timeouts_blocked ());

  /* accept/not accept file protocol & consider/not consider FILE 
   * protocol like a error */
  WGET ("radio_file_protocol", g, widget);
  if (uc_project_get_proto_file_check ())
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), TRUE);
  else
    {
      WGET ("radio_file_is_error", g, widget);
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), TRUE);
    }

  /* accept/not accept cookies */
  WGET ("radio_cookies_accept", g, widget);
  g_signal_connect (G_OBJECT (widget), "toggled",
		    G_CALLBACK (uc_application_settings_cookies_toggled_cb),
		    NULL);
  if (uc_project_get_cookies_accept ())
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), TRUE);

  WGET ("radio_cookies_none", g, widget);
  g_signal_connect (G_OBJECT (widget), "toggled",
		    G_CALLBACK (uc_application_settings_cookies_toggled_cb),
		    NULL);
  if (!uc_project_get_cookies_accept ())
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), TRUE);

  /* email check */
  WGET ("check_email_address", g, widget);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget),
				(gboolean) uc_project_get_proto_mailto ());

  /* warn for cookies */
  WGET ("check_cookies_warn_added", g, widget);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget),
				(gboolean)
				uc_project_get_cookies_warn_added ());
  WGET ("check_cookies_warn_updated", g, widget);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget),
				(gboolean)
				uc_project_get_cookies_warn_updated ());
  WGET ("check_cookies_warn_deleted", g, widget);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget),
				(gboolean)
				uc_project_get_cookies_warn_deleted ());

  /* email MX check */
  WGET ("check_email_mx", g, widget);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget),
				(gboolean)
				uc_project_get_proto_mailto_check_mx ());

  gtk_widget_set_sensitive (GTK_WIDGET (widget),
			    (gboolean) uc_project_get_proto_mailto ());

  /* FTP check */
  WGET ("check_ftp_protocol", g, widget);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget),
				(gboolean) uc_project_get_proto_ftp ());

  WGET ("use_passive_ftp", g, widget);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget),
				(gboolean) uc_project_get_passive_ftp ());

  /* HTTPS check */
  WGET ("check_https_protocol", g, widget);
#ifdef ENABLE_GNUTLS
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget),
				(gboolean) uc_project_get_proto_https ());
#else
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), FALSE);
  WSENS (g, "check_https_protocol", FALSE);
#endif

  gtk_widget_show_all (settings_dialog);
}

/**
 * uc_application_settings_get_data:
 * 
 * Retrieve user input data settings.
 *
 * Returns: TRUE if all inputs are ok.
 */
gboolean
uc_application_settings_get_data (void)
{
  GtkWidget *widget = NULL;
  GladeXML *g = glade_settings_dialog;

  /* network timeout */
  WGET ("timeout", g, widget);
  uc_project_set_check_timeout ((gint)
				gtk_spin_button_get_value (GTK_SPIN_BUTTON
							   (widget)));

  /* prompt for auth */
  WGET ("prompt_auth", g, widget);
  uc_project_set_prompt_auth ((guint)
			      gtk_toggle_button_get_active
			      (GTK_TOGGLE_BUTTON (widget)));

  /* active debug mode? */
  WGET ("debug_mode", g, widget);
  uc_project_set_debug_mode ((guint)
			     gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
							   (widget)));

  /* dump properties? */
  WGET ("dump_properties", g, widget);
  uc_project_set_dump_properties ((guint)
				  gtk_toggle_button_get_active
				  (GTK_TOGGLE_BUTTON (widget)));

  /* networking depth level */
  WGET ("url_depth_level", g, widget);
  uc_project_set_depth_level ((gint)
			      gtk_spin_button_get_value (GTK_SPIN_BUTTON
							 (widget)));

  /* retrieving images content */
  WGET ("images_content", g, widget);
  uc_project_set_download_images_content ((guint)
					  gtk_toggle_button_get_active
					  (GTK_TOGGLE_BUTTON (widget)));

  /* retrieving archives content */
  WGET ("archives_content", g, widget);
  uc_project_set_download_archives_content ((guint)
					    gtk_toggle_button_get_active
					    (GTK_TOGGLE_BUTTON (widget)));

  /* networking check wait */
  WGET ("url_wait", g, widget);
  uc_project_set_check_wait ((gint)
			     gtk_spin_button_get_value (GTK_SPIN_BUTTON
							(widget)));

  /* documents to reject */
  WGET ("frame_filters_documents", g, widget);
  uc_application_buttons_save_setting (widget, "documents");

  /* images to reject */
  WGET ("frame_filters_images", g, widget);
  uc_application_buttons_save_setting (widget, "images");

  /* directories filter */
  uc_application_filters_save_settings (treeview_filter_directories,
					"directories");

  /* domains filter */
  uc_application_filters_save_settings (treeview_filter_domains, "domains");

  /* value for domains in timeout */
  WGET ("timeouts_blocked", g, widget);
  uc_project_set_timeouts_blocked ((gint)
				   gtk_spin_button_get_value (GTK_SPIN_BUTTON
							      (widget)));

  /* check the FILE protocol & accept/not accept FILE protocol */
  WGET ("radio_file_protocol", g, widget);
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)))
    {
      uc_project_set_proto_file_check (1);
      uc_project_set_proto_file_is_error (0);
    }
  else
    {
      uc_project_set_proto_file_check (0);
      uc_project_set_proto_file_is_error (1);
    }

  /* accept/not accept cookies */
  WGET ("radio_cookies_accept", g, widget);
  uc_project_set_cookies_accept (gtk_toggle_button_get_active
				 (GTK_TOGGLE_BUTTON (widget)));

  /* check emails */
  WGET ("check_email_address", g, widget);
  uc_project_set_proto_mailto ((guint)
			       gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
							     (widget)));

  /* warn for cookies */
  WGET ("check_cookies_warn_added", g, widget);
  uc_project_set_cookies_warn_added ((guint)
				     gtk_toggle_button_get_active
				     (GTK_TOGGLE_BUTTON (widget)));
  WGET ("check_cookies_warn_updated", g, widget);
  uc_project_set_cookies_warn_updated ((guint)
				       gtk_toggle_button_get_active
				       (GTK_TOGGLE_BUTTON (widget)));
  WGET ("check_cookies_warn_deleted", g, widget);
  uc_project_set_cookies_warn_deleted ((guint)
				       gtk_toggle_button_get_active
				       (GTK_TOGGLE_BUTTON (widget)));

  /* check MX emails */
  WGET ("check_email_mx", g, widget);
  uc_project_set_proto_mailto_check_mx ((guint)
					gtk_toggle_button_get_active
					(GTK_TOGGLE_BUTTON (widget)));

  /* check FTP */
  WGET ("check_ftp_protocol", g, widget);
  uc_project_set_proto_ftp ((guint)
			    gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
							  (widget)));

  WGET ("use_passive_ftp", g, widget);
  uc_project_set_passive_ftp ((guint)
			      gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
							    (widget)));

  /* check HTTPS */
  WGET ("check_https_protocol", g, widget);
  uc_project_set_proto_https ((guint)
			      gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
							    (widget)));


  uc_project_general_settings_save ();

  /* FIXME */
  if (uc_project_get_url ())
    uc_project_set_save (TRUE);

  return TRUE;
}

/**
 * uc_application_filters_save_settings:
 * @treeview: Tree view to work with.
 * @path: Filter type ("domains" or "directories").
 * 
 * Save filters settings.
 */
static void
uc_application_filters_save_settings (GtkTreeView * treeview,
				      const gchar * path)
{
  GtkTreeModel *tree_store = NULL;
  gchar **filters_vector = NULL;
  GtkTreeIter iter;
  GString *filters = g_string_new ("");
  guint size = 0;

  tree_store = gtk_tree_view_get_model (treeview);
  if (gtk_tree_model_get_iter_first (tree_store, &iter))
    do
      {
	GValue *value = NULL;

	value = g_new0 (GValue, 1);
	gtk_tree_model_get_value (tree_store, &iter, 0, value);

	g_string_append_printf (filters, ",%s",
				(gchar *) g_value_get_string (value));

	g_value_unset (value);
	size++;
      }
    while (gtk_tree_model_iter_next (tree_store, &iter));

  if (size > 0)
    g_string_erase (filters, 0, 1);

  filters_vector = g_strsplit (filters->str, ",", -1);

  if (!strcmp (path, "directories"))
    uc_project_set_reject_directories (filters->str);
  else if (!strcmp (path, "domains"))
    uc_project_set_reject_domains (filters->str);

  g_strfreev (filters_vector), filters_vector = NULL;
  g_string_free (filters, TRUE), filters = NULL;
}

/**
 * uc_application_progress_dialog_show:
 * 
 * Show the progess dialog.
 */
void
uc_application_progress_dialog_show (void)
{
  GtkWidget *widget = NULL;
  GladeXML *g = NULL;

  WINIT ("progress_dialog", glade_progress_dialog, progress_dialog);
  g = glade_progress_dialog;

  WGET ("dynamic_values_time_between", g, widget);
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (widget),
			     (gfloat) uc_project_get_check_wait ());

  WGET ("dynamic_values_timeout", g, widget);
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (widget),
			     (gfloat) uc_project_get_check_timeout ());

  gtk_widget_show_all (progress_dialog);
}

/**
 * uc_application_about_dialog_show:
 * 
 * Show the about box.
 */
void
uc_application_about_dialog_show (void)
{
  GtkWidget *about_dialog = NULL;
  GtkWidget *hbox = NULL;
  GtkWidget *href = NULL;
  GdkPixbuf *pixbuf = NULL;
  const gchar *authors[] = {
    _("Emmanuel Saracco <esaracco@users.labs.libre-entreprise.org>, main developer\n" "Stephane Loeuillet <leroutier@users.labs.libre-entreprise.org>, co-developer\n" "\n" "Graphic arts:\n" "Jan Henrik Helmers <jh@helmers.org>\n" "Emmanuel Saracco <esaracco@users.labs.libre-entreprise.org>"),
    NULL
  };
  const gchar *documentors[] = {
    _("Manpage:"),
    "Daniel Pecos <dani@netpecos.org>",
    "Emmanuel Saracco <esaracco@users.labs.libre-entreprise.org>",
    "",
    _("Code:"),
    "Emmanuel Saracco <esaracco@users.labs.libre-entreprise.org>",
    NULL
  };
  gchar *translators =
    _("Takeshi Aihana <aihana@gnome.gr.jp>, Japanese\n"
      "Alexander Hunziker <alex.hunziker@rega-sense.ch>, German\n"
      "Florent Jugla <juglaf@wanadoo.fr>, Esperanto\n"
      "Daniel Pecos <dani@netpecos.org>, Spanish\n"
      "Gunner Poulsen <gp@lyngbjerggaardskolen.dk>, Danish\n"
      "Emmanuel Saracco <esaracco@users.labs.libre-entreprise.org>, French\n"
      "Angelo Theodorou <angeloth@tin.it>, Italian\n");

  pixbuf =
    gdk_pixbuf_new_from_file (UC_PIXMAPS_DIR "/gurlchecker_icon.png", NULL);

  about_dialog =
    gnome_about_new ("gURLChecker",
		     UC_VERSION,
		     "(C) 2002-2005 Emmanuel Saracco <esaracco@users.labs.libre-entreprise.org>",
		     _("gURLChecker is a graphical web links checker.\n"
		       "It can work on a whole site, a single local page "
		       "or a browser bookmarks file"),
		     authors, documentors, translators, pixbuf);

  hbox = gtk_hbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (about_dialog)->vbox),
		      hbox, FALSE, FALSE, 0);
  href =
    gnome_href_new ("http://labs.libre-entreprise.org/projects/gurlchecker/",
		    _("gurlchecker project page"));
  gtk_box_pack_start (GTK_BOX (hbox), href, TRUE, FALSE, 0);

  g_object_unref (pixbuf), pixbuf = NULL;

  gtk_window_set_destroy_with_parent (GTK_WINDOW (about_dialog), TRUE);
  gtk_widget_show_all (about_dialog);
}

/**
 * uc_application_dialog_yes_no_show:
 * @message: The message to display.
 * @msg_type: Type of the message.
 * 
 * Print a yes/no choice dialog.
 *  
 * Returns: TRUE if user clicked on the OK button.
 */
gboolean
uc_application_dialog_yes_no_show (const gchar * message,
				   const GtkMessageType msg_type)
{
  gint ret = 0;

  GtkWidget *dialog = gtk_message_dialog_new (NULL,
					      GTK_DIALOG_MODAL |
					      GTK_DIALOG_DESTROY_WITH_PARENT,
					      msg_type,
					      GTK_BUTTONS_YES_NO,
					      message);

  gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
  gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
  ret = gtk_dialog_run (GTK_DIALOG (dialog));
  gtk_widget_destroy (dialog), dialog = NULL;

  return (ret == GTK_RESPONSE_YES);
}

/**
 * uc_application_dialog_show:
 * @message: The message to display.
 * @msg_type: Type of the message.
 * 
 * Print a alert box.
 */
void
uc_application_dialog_show (const gchar * message,
			    const GtkMessageType msg_type)
{
  GtkWidget *dialog = gtk_message_dialog_new (NULL,
					      GTK_DIALOG_MODAL |
					      GTK_DIALOG_DESTROY_WITH_PARENT,
					      msg_type,
					      GTK_BUTTONS_OK,
					      message);

  gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
  gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
  gtk_dialog_run (GTK_DIALOG (dialog));
  gtk_widget_destroy (dialog), dialog = NULL;
}

/**
 * uc_application_new_local_file_dialog_show:
 * 
 * Show the new local file project dialog box.
 */
void
uc_application_new_local_file_dialog_show (void)
{
  GtkWidget *widget = NULL;

  if (uc_project_get_save () &&
      uc_application_dialog_yes_no_show (_("Save the current project?"),
					 GTK_MESSAGE_QUESTION))
    uc_project_save ();
  else if (uc_project_get_save_bookmarks ())
    uc_application_save_bookmarks ();

  WINIT ("new_local_file_project_dialog", glade_new_project_dialog,
	 new_project_dialog);

  WGET ("project_url", glade_new_project_dialog, widget);

  if (uc_project_get_url () != NULL)
    gtk_entry_set_text (GTK_ENTRY
			(gnome_file_entry_gtk_entry
			 (GNOME_FILE_ENTRY (widget))), uc_project_get_url ());

  gtk_widget_show_all (new_project_dialog);
}

/**
 * uc_application_new_web_dialog_show:
 * 
 * Show the new web project dialog box.
 */
void
uc_application_new_web_dialog_show (void)
{
  GtkWidget *widget = NULL;

  if (uc_project_get_save () &&
      uc_application_dialog_yes_no_show (_("Save the current project?"),
					 GTK_MESSAGE_QUESTION))
    uc_project_save ();
  else if (uc_project_get_save_bookmarks ())
    uc_application_save_bookmarks ();

  WINIT ("new_web_project_dialog", glade_new_project_dialog,
	 new_project_dialog);

  WGET ("project_url", glade_new_project_dialog, widget);

  if (uc_project_get_url () != NULL)
    gtk_entry_set_text (GTK_ENTRY
			(gnome_entry_gtk_entry (GNOME_ENTRY (widget))),
			uc_project_get_url ());

  gtk_widget_show_all (new_project_dialog);
}

/**
 * uc_application_new_bookmarks_dialog_show:
 * 
 * Show the new bookmarks project dialog box.
 */
void
uc_application_new_bookmarks_dialog_show (void)
{
  GtkWidget *widget = NULL;

  if (uc_project_get_save () &&
      uc_application_dialog_yes_no_show (_("Save the current project?"),
					 GTK_MESSAGE_QUESTION))
    uc_project_save ();
  else if (uc_project_get_save_bookmarks ())
    uc_application_save_bookmarks ();

  WINIT ("new_bookmarks_project_dialog", glade_new_project_dialog,
	 new_project_dialog);

  WGET ("project_bookmarks_file", glade_new_project_dialog, widget);

  if (uc_project_get_bookmarks_file () != NULL)
    gnome_file_entry_set_default_path (GNOME_FILE_ENTRY (widget),
				       uc_project_get_bookmarks_file ());

  gtk_widget_show_all (new_project_dialog);
}

/**
 * uc_application_add_filter_domain_add:
 * 
 * Add a domain filter.
 */
void
uc_application_add_filter_domain_add (void)
{
  GtkWidget *entry = NULL;
  gchar *tmp = NULL;

  WGET ("entry_add_filter_domain", glade_add_filter_domain_dialog, entry);

  tmp = (gchar *) gtk_entry_get_text (GTK_ENTRY (entry));
  if (tmp != NULL && g_strstrip (tmp) && strlen (tmp) > 0)
    {
      GtkTreeIter iter;

      /* very basic check of the domain name syntax */
      if ((strlen (tmp) < 4) ||
	  (!strchr (tmp, '.')) ||
	  (tmp[strlen (tmp) - 1] == '.') ||
	  (tmp[strlen (tmp) - 2] == '.') || (tmp[0] == '.'))
	uc_application_dialog_show (_
				    ("The domain you entered "
				     "seems not to be valid."),
				    GTK_MESSAGE_ERROR);
      else
	{
	  gtk_list_store_append (treestore_filter_domains, &iter);
	  gtk_list_store_set (treestore_filter_domains, &iter, 0, tmp, -1);
	  gtk_widget_destroy (add_filter_domain_dialog),
	    add_filter_domain_dialog = NULL;
	}
    }
  else
    uc_application_dialog_show (_
				("Please, enter a domain name."),
				GTK_MESSAGE_ERROR);
}

/**
 * uc_application_treeview_get_selected_iter:
 * @tv: Tree view to work with.
 * @iter: Iterater to fill.
 * 
 * set the iterator for the current
 * selection in a given treeview.
 *
 * Returns: TRUE if it have filled the given iter with selection.
 */
gboolean
uc_application_treeview_get_selected_iter (const GtkTreeView * tv,
					   GtkTreeIter * iter)
{
  GtkTreeModel *model = NULL;
  GtkTreeSelection *select = gtk_tree_view_get_selection (GTK_TREE_VIEW (tv));
  gint32 ret = -1;

  if (gtk_tree_selection_get_selected (select, &model, iter))
    {
      gtk_tree_model_get (model, iter, 0, &ret, -1);

      return TRUE;
    }

  return FALSE;
}

/**
 * uc_application_add_filter_domain_remove:
 * 
 * Remove a domain filter.
 */
void
uc_application_add_filter_domain_remove (void)
{
  GtkTreeIter iter;

  if (uc_application_treeview_get_selected_iter
      (treeview_filter_domains, &iter))
    gtk_list_store_remove (treestore_filter_domains, &iter);
  else
    uc_application_dialog_show (_
				("Please, select a item first."),
				GTK_MESSAGE_ERROR);
}

/**
 * uc_application_add_filter_directory_add:
 * 
 * Add a directory filter.
 */
void
uc_application_add_filter_directory_add (void)
{
  GtkWidget *entry = NULL;
  gchar *tmp = NULL;

  WGET ("entry_add_filter_directory", glade_add_filter_directory_dialog,
	entry);

  tmp = (gchar *) gtk_entry_get_text (GTK_ENTRY (entry));
  if (tmp != NULL && g_strstrip (tmp) && strlen (tmp) > 0)
    {
      GtkTreeIter iter;
      gchar *tmp_path = NULL;

      /* force directory to be "slashed" */
      if (tmp[strlen (tmp) - 1] == '/')
	tmp[strlen (tmp) - 1] = '\0';

      tmp_path = (tmp[0] == '/') ?
	g_strconcat (tmp, "/", NULL) : g_strconcat ("/", tmp, "/", NULL);

      gtk_list_store_append (treestore_filter_directories, &iter);
      gtk_list_store_set (treestore_filter_directories, &iter, 0, tmp_path,
			  -1);
      gtk_widget_destroy (add_filter_directory_dialog),
	add_filter_directory_dialog = NULL;
      g_free (tmp_path), tmp_path = NULL;
    }
  else
    uc_application_dialog_show (_
				("Please, enter a directory name."),
				GTK_MESSAGE_ERROR);
}

/**
 * uc_application_add_filter_directory_remove:
 * 
 * Remove a directory filter.
 */
void
uc_application_add_filter_directory_remove (void)
{
  GtkTreeIter iter;

  if (uc_application_treeview_get_selected_iter
      (treeview_filter_directories, &iter))
    gtk_list_store_remove (treestore_filter_directories, &iter);
  else
    uc_application_dialog_show (_
				("Please, select a item first."),
				GTK_MESSAGE_ERROR);
}

/**
 * uc_application_search_get_data:
 * 
 * Retreive the string to search and the search
 * properties.
 */
void
uc_application_search_get_data (void)
{
  GtkWidget *widget = NULL;
  gchar *tmp = NULL;

  WGET ("search_string", glade_new_search_dialog, widget);
  tmp = (gchar *)
    gtk_entry_get_text (GTK_ENTRY
			(gnome_entry_gtk_entry (GNOME_ENTRY (widget))));

  if (tmp != NULL)
    {
      g_strstrip (tmp);

      if (strlen (tmp) > 0)
	{
	  gnome_entry_append_history (GNOME_ENTRY (widget), TRUE, tmp);

	  uc_search_free ();
	  search_state.string = g_strdup (tmp);

	  /* FIXME
	     search_state.case_sensitive =
	     (gtk_toggle_button_get_active
	     (GTK_TOGGLE_BUTTON
	     (glade_xml_get_widget
	     (glade_new_search_dialog, "search_case sensitive"))));
	   */
	  search_state.case_sensitive = TRUE;

	  search_state.pages_content =
	    (gtk_toggle_button_get_active
	     (GTK_TOGGLE_BUTTON
	      (glade_xml_get_widget
	       (glade_new_search_dialog, "search_pages_content"))));
	  search_state.links_name =
	    (gtk_toggle_button_get_active
	     (GTK_TOGGLE_BUTTON
	      (glade_xml_get_widget
	       (glade_new_search_dialog, "search_links_name"))));
	  search_state.meta_content =
	    (gtk_toggle_button_get_active
	     (GTK_TOGGLE_BUTTON
	      (glade_xml_get_widget
	       (glade_new_search_dialog, "search_meta_content"))));
	  search_state.emails =
	    (gtk_toggle_button_get_active
	     (GTK_TOGGLE_BUTTON
	      (glade_xml_get_widget
	       (glade_new_search_dialog, "search_emails"))));

	  gtk_widget_destroy (new_search_dialog), new_search_dialog = NULL;
	  UC_UPDATE_UI;
	  uc_search_begin ();

	  return;
	}
    }

  uc_application_dialog_show (_("Please, enter a string to search for."),
			      GTK_MESSAGE_ERROR);
}

/**
 * uc_application_project_get_data:
 * @type: FIXME.
 * 
 * Retrieving url to check and some check
 * properties.
 *
 * Returns: %TRUE if data input is ok.
 */
gboolean
uc_application_project_get_data (const UCProjectType type)
{
  GtkWidget *widget = NULL;
  gchar *tmp = NULL;

  WGET ("project_url", glade_new_project_dialog, widget);
  if (type == UC_PROJECT_TYPE_WEB_SITE)
    tmp = (gchar *)
      gtk_entry_get_text (GTK_ENTRY
			  (gnome_entry_gtk_entry (GNOME_ENTRY (widget))));
  else
    tmp = (gchar *)
      gtk_entry_get_text (GTK_ENTRY
			  (gnome_file_entry_gtk_entry
			   (GNOME_FILE_ENTRY (widget))));

  if (tmp != NULL)
    {
      g_strstrip (tmp);

      if (strlen (tmp) > 0)
	{
	  gchar *tmp_url = NULL;

	  tmp_url = uc_url_add_protocol ((type == UC_PROJECT_TYPE_WEB_SITE) ?
					 UC_PROTOCOL_HTTP : UC_PROTOCOL_FILE,
					 tmp);

	  if (uc_url_is_valid (tmp_url))
	    {
	      gchar *auth_user = NULL;
	      gchar *auth_password = NULL;
	      gchar *auth_line = NULL;

	      /* add the given url in the combo history */
	      if (type == UC_PROJECT_TYPE_WEB_SITE)
		{
		  gnome_entry_append_history (GNOME_ENTRY (widget), TRUE,
					      tmp);

		  /* manage authentication */
		  uc_project_set_auth_user (NULL);
		  uc_project_set_auth_password (NULL);
		  uc_project_set_auth_line (NULL);
		  if (uc_utils_get_auth_fields (glade_new_project_dialog,
						&auth_user, &auth_password,
						&auth_line))
		    {
		      uc_project_set_auth_user (auth_user);
		      uc_project_set_auth_password (auth_password);
		      uc_project_set_auth_line (auth_line);
		      g_free (auth_line), auth_line = NULL;
		    }

		  /* ignore URLs arguments */
		  widget = glade_xml_get_widget
		    (glade_new_project_dialog, "no_urls_args");
		  uc_project_set_no_urls_args ((gboolean)
					       gtk_toggle_button_get_active
					       (GTK_TOGGLE_BUTTON (widget)));
		}
	      else
		{
		  gnome_entry_append_history (GNOME_ENTRY
					      (gnome_file_entry_gnome_entry
					       (GNOME_FILE_ENTRY (widget))),
					      TRUE, tmp);

		  /* force check for files, even if user chose "consider as 
		   * error" in preferences */
		  uc_utils_swap_file_proto_option (TRUE);
		}

	      /* do not retrieve content (speed up the check) */
	      widget = glade_xml_get_widget
		(glade_new_project_dialog, "speed_check");
	      uc_project_set_speed_check ((guint)
					  gtk_toggle_button_get_active
					  (GTK_TOGGLE_BUTTON (widget)));

	      /* only check this directory and its subdirectories? */
	      widget = glade_xml_get_widget
		(glade_new_project_dialog, "check_chroot");
	      uc_project_set_check_chroot ((guint)
					   gtk_toggle_button_get_active
					   (GTK_TOGGLE_BUTTON (widget)));

	      /* only check local links */
	      widget = glade_xml_get_widget
		(glade_new_project_dialog, "limit_local");
	      uc_project_set_limit_local ((guint)
					  gtk_toggle_button_get_active
					  (GTK_TOGGLE_BUTTON (widget)));

	      uc_project_set_url (tmp_url);

	      uc_project_new (0);
	      uc_project_set_type (type);
	      gtk_widget_destroy (new_project_dialog),
		new_project_dialog = NULL;
	      UC_UPDATE_UI;
	      uc_web_site_begin_check ();

	      uc_project_set_no_urls_args (FALSE);
	      uc_project_set_speed_check (FALSE);
	      uc_project_set_check_chroot (FALSE);
	      uc_project_set_limit_local (FALSE);

	      /* restore original options for file protocol */
	      if (uc_project_get_type () == UC_PROJECT_TYPE_LOCAL_FILE)
		uc_utils_swap_file_proto_option (FALSE);

	      return TRUE;
	    }

	  g_free (tmp_url), tmp_url = NULL;
	}
    }

  uc_application_dialog_show (_("Please, check that you "
				"entered the URL correctly and try again."),
			      GTK_MESSAGE_ERROR);

  return FALSE;
}

/**
 * uc_application_get_bookmarks_project_data:
 * 
 * Retrieving bookmarks file to check.
 */
void
uc_application_get_bookmarks_project_data (void)
{
  GtkWidget *widget = NULL;
  gchar *file = NULL;

  WGET ("project_bookmarks_file", glade_new_project_dialog, widget);

  file = gnome_file_entry_get_full_path (GNOME_FILE_ENTRY (widget), FALSE);
  if (file == NULL)
    {
      uc_application_dialog_show (_
				  ("Please choose a bookmarks file to check."),
				  GTK_MESSAGE_ERROR);
      return;
    }

  if (!g_file_test (file, G_FILE_TEST_EXISTS))
    {
      uc_application_dialog_show (_
				  ("The file you entered is not correct "
				   "or your don't have appropriate "
				   "rights on it. Please choose a valid "
				   "bookmarks file."), GTK_MESSAGE_ERROR);
      return;
    }

  /* for the moment we accept only XBEL format */
  if (!uc_bookmarks_format_is_xbel (file))
    {
      uc_application_dialog_show (_
				  ("The format of the selected file is not "
				   "correct.\n"
				   "For the moment, gURLChecker can deal "
				   "only with XBEL format."),
				  GTK_MESSAGE_ERROR);

      return;
    }

  gnome_entry_append_history (GNOME_ENTRY
			      (gnome_file_entry_gnome_entry
			       (GNOME_FILE_ENTRY (widget))), TRUE, file);

  /* do not retrieve content (speed up the check) */
  widget = glade_xml_get_widget (glade_new_project_dialog, "speed_check");
  uc_project_set_speed_check ((guint)
			      gtk_toggle_button_get_active
			      (GTK_TOGGLE_BUTTON (widget)));

  uc_project_set_bookmarks_file (file);

  uc_project_new (0);
  gtk_widget_destroy (new_project_dialog), new_project_dialog = NULL;
  UC_UPDATE_UI;
  uc_bookmarks_begin_check ();

  uc_project_set_save (TRUE);

  /* FIXME
   * for the moment we are not able to manage projects
   * for bookmarks files...
   * */
  WSENS (glade_main_window, "project_properties", FALSE);
}

/**
 * uc_application_draw_main_frames:
 * 
 * Draw the two principals main frames
 * for displaying urls list and headers.
 */
void
uc_application_draw_main_frames (void)
{
  GtkWidget *main_frames = NULL;
  GtkWidget *right_frames = NULL;
  GtkWidget *urls_list = NULL;
  GtkWidget *scrolled_window = NULL;

  WGET ("main_frames", glade_main_window, main_frames);
  WGET ("right_frames", glade_main_window, right_frames);
  WGET ("scrolled_window", glade_main_window, scrolled_window);

  UC_GET_WIDGET ("urls_list", main_window, urls_list);
  if (urls_list != NULL)
    gtk_widget_destroy (urls_list), urls_list = NULL;

  urls_list = gtk_tree_view_new ();
  gtk_widget_show_all (urls_list);
  gtk_container_add (GTK_CONTAINER (scrolled_window), urls_list);
  UC_SET_WIDGET ("urls_list", main_window, urls_list);

  g_signal_connect (G_OBJECT (urls_list), "cursor_changed",
		    G_CALLBACK (on_urls_list_cursor_changed), NULL);

  g_signal_connect (G_OBJECT (urls_list), "button_press_event",
		    G_CALLBACK (on_url_list_mouse_clicked), NULL);

  g_signal_connect (G_OBJECT (urls_list), "button_release_event",
		    G_CALLBACK (on_url_list_mouse_clicked), NULL);

  g_signal_connect (G_OBJECT (urls_list),
		    "motion-notify-event",
		    G_CALLBACK (on_main_treeview_motion_notify_event), NULL);

  g_signal_connect (G_OBJECT (urls_list),
		    "leave_notify_event",
		    G_CALLBACK (on_main_treeview_leave_notify_event), NULL);

  g_signal_connect (G_OBJECT (urls_list),
		    "enter_notify_event",
		    G_CALLBACK (on_main_treeview_enter_notify_event), NULL);


  gtk_widget_show_all (right_frames);
  gtk_widget_show_all (main_frames);
}

/**
 * uc_application_get_status_code_properties:
 * @status_code: Status code to work with.
 * 
 * Return properties for a given status code.
 *
 * Returns: #UCStatusCode propertie node associated with the given status code.
 */
UCStatusCode *
uc_application_get_status_code_properties (const gchar * status_code)
{
  UCStatusCode *sc = NULL;

  if (status_code != NULL)
    sc = g_hash_table_lookup
      (uc_application_status_code_properties_hash, status_code);

  if (sc == NULL)
    sc = g_hash_table_lookup (uc_application_status_code_properties_hash,
			      UC_STATUS_CODE_UNKNOWN);

  return sc;
}

/**
 * uc_application_quit:
 * 
 * Do a clean application exit.
 */
void
uc_application_quit (void)
{
  /* FIXME */
  if (uc_project_get_save_bookmarks ())
    uc_application_save_bookmarks ();
  else if (uc_project_get_save () &&
	   uc_application_dialog_yes_no_show (_("Save the current project?"),
					      GTK_MESSAGE_QUESTION))
    uc_project_save ();

  uc_application_clean_data ();

  gtk_exit (0);
}

/**
 * uc_application_clean_data:
 * 
 * General clean up routine call at application
 * exit.
 */
static void
uc_application_clean_data (void)
{
  uc_application_remove_paths ();
  uc_project_free (UC_PROJECT_TYPE_ALL);
  uc_search_free ();
  uc_lists_checked_links_free ();
  uc_lists_already_checked_free ();
  uc_timeout_domains_free ();
  uc_application_status_code_properties_free ();

  if (treestore_filter_directories != NULL)
    g_object_unref (treestore_filter_directories),
      treestore_filter_directories = NULL;
  if (treestore_filter_domains != NULL)
    g_object_unref (treestore_filter_domains), treestore_filter_domains =
      NULL;
  if (treestore_search != NULL)
    g_object_unref (treestore_search), treestore_search = NULL;
  if (treestore != NULL)
    g_object_unref (treestore), treestore = NULL;
}

/**
 * uc_application_globals_init:
 *
 * Initialisation of all globals.
 */
void
uc_application_globals_init (void)
{
/* global pointers for glade xml file */
  glade_main_window = NULL;
  glade_settings_dialog = NULL;
  glade_new_project_dialog = NULL;
  glade_progress_dialog = NULL;
  glade_view_source_dialog = NULL;
  glade_view_image_dialog = NULL;
  glade_similar_links_dialog = NULL;
  glade_page_information_dialog = NULL;
  glade_add_filter_directory_dialog = NULL;
  glade_add_filter_domain_dialog = NULL;
  glade_new_search_dialog = NULL;
  glade_search_dialog = NULL;
  glade_save_bookmarks_dialog = NULL;
  glade_project_information_dialog = NULL;
  glade_open_project_dialog = NULL;

  /* global windows and dialogs pointers */
  main_window = NULL;
  new_project_dialog = NULL;
  settings_dialog = NULL;
  progress_dialog = NULL;
  view_source_dialog = NULL;
  view_image_dialog = NULL;
  similar_links_dialog = NULL;
  page_information_dialog = NULL;
  add_filter_directory_dialog = NULL;
  add_filter_domain_dialog = NULL;
  new_search_dialog = NULL;
  search_dialog = NULL;
  save_bookmarks_dialog = NULL;
  project_information_dialog = NULL;
  open_project_dialog = NULL;

  /* treeview pointers */
  treeview = NULL;
  treeview_filter_directories = NULL;
  treeview_projects = NULL;
  treeview_filter_domains = NULL;
  treeview_search = NULL;

  /* treestore for the url treeview list */
  treestore = NULL;
  treestore_filter_directories = NULL;
  treestore_projects = NULL;
  treestore_filter_domains = NULL;
  treestore_search = NULL;

  uc_refresh_page = FALSE;
}

/**
 * uc_application_init:
 * @url: the command line URL, or %NULL
 * @auth_user: user for authentication.
 * @auth_password: password for authentication.
 * 
 * gURLChecker initialization.
 */
void
uc_application_init (gchar * url, gchar * auth_user, gchar * auth_password,
		     gboolean no_urls_args)
{
  uc_conn_init ();
  uc_timeout_domains_init ();
  uc_tooltips_init ();
  uc_application_menu_init ();
  uc_application_status_code_properties_init ();
  uc_check_reset ();
  uc_application_settings_init ();
  uc_project_new (0);
  uc_application_draw_main_frames ();
  UC_UPDATE_UI;

  /* if user specified a url to check on
   * command line, go and check it */
  if (url)
    {
      uc_project_set_url (url);
      if (auth_user && auth_password)
	{
	  gchar *auth_line = NULL;

	  uc_project_set_auth_user (auth_user);
	  uc_project_set_auth_password (auth_password);
	  auth_line = uc_utils_build_auth_line (auth_user, auth_password);
	  uc_project_set_auth_line (auth_line);
	  g_free (auth_line), auth_line = NULL;
	}
      uc_project_set_no_urls_args (no_urls_args);
      uc_web_site_begin_check ();
    }
}

/**
 * uc_application_make_paths:
 * 
 * Create the directories needed by gURLChecker.
 */
void
uc_application_make_paths (void)
{
  gchar *path = NULL;

  path = g_strconcat (uc_project_get_working_path (), "/",
		      uc_project_get_cache_name (), NULL);
  uc_utils_mkdirs (path, TRUE);
  g_free (path), path = NULL;

  path = g_strconcat (uc_project_get_working_path (), "/projects", NULL);
  uc_utils_mkdirs (path, TRUE);
  g_free (path), path = NULL;
}

/**
 * uc_application_settings_init:
 * 
 * Set the defaults properties the first time
 * gURLChecker is launched.
 */
static void
uc_application_settings_init (void)
{
  uc_project_xml_load_settings (NULL);
}

/**
 * uc_application_menu_init:
 * 
 * First menu items management.
 */
static void
uc_application_menu_init (void)
{
  /* no treeview, then no display options */
  WSENS (glade_main_window, "display", FALSE);
  WSENS (glade_main_window, "find", FALSE);

  uc_project_set_save_bookmarks (FALSE);
  uc_project_set_save (FALSE);
  WSENS (glade_main_window, "delete_project", FALSE);

  WSENS (glade_main_window, "refresh_all", FALSE);
  WSENS (glade_main_window, "refresh_main_page", FALSE);

#ifndef ENABLE_GNUTLS
  WSENS (glade_main_window, "https_protocol", FALSE);
#endif

  WSENS (glade_main_window, "project_properties", FALSE);
}

/**
 * uc_application_set_status_bar:
 * @progress: 0, or a value if we want progress bar on the
 *            status bar.
 * @msg: message to display in satus bar.
 * 
 * Refresh application status bar.
 */
void
uc_application_set_status_bar (const gfloat progress, const gchar * msg)
{
  GtkWidget *widget = NULL;

  WGET ("status_bar", glade_main_window, widget);

  if (progress > 0)
    gnome_appbar_set_progress_percentage (GNOME_APPBAR (widget), progress);
  if (msg != NULL)
    gnome_appbar_push (GNOME_APPBAR (widget), msg);
}
